from ._data import (
    Binarizer as Binarizer,
    KernelCenterer as KernelCenterer,
    MinMaxScaler as MinMaxScaler,
    MaxAbsScaler as MaxAbsScaler,
    Normalizer as Normalizer,
    RobustScaler as RobustScaler,
    StandardScaler as StandardScaler,
    QuantileTransformer as QuantileTransformer,
    add_dummy_feature as add_dummy_feature,
    binarize as binarize,
    normalize as normalize,
    scale as scale,
    robust_scale as robust_scale,
    maxabs_scale as maxabs_scale,
    minmax_scale as minmax_scale,
    quantile_transform as quantile_transform,
    power_transform as power_transform,
    PowerTransformer as PowerTransformer,
)
from ._encoders import OneHotEncoder as OneHotEncoder, OrdinalEncoder as OrdinalEncoder
from ._function_transformer import FunctionTransformer as FunctionTransformer
from ._polynomial import (
    PolynomialFeatures as PolynomialFeatures,
    SplineTransformer as SplineTransformer,
)
from ._label import (
    label_binarize as label_binarize,
    LabelBinarizer as LabelBinarizer,
    LabelEncoder as LabelEncoder,
    MultiLabelBinarizer as MultiLabelBinarizer,
)
from ._discretization import KBinsDiscretizer as KBinsDiscretizer


__all__ = [
    "Binarizer",
    "FunctionTransformer",
    "KBinsDiscretizer",
    "KernelCenterer",
    "LabelBinarizer",
    "LabelEncoder",
    "MultiLabelBinarizer",
    "MinMaxScaler",
    "MaxAbsScaler",
    "QuantileTransformer",
    "Normalizer",
    "OneHotEncoder",
    "OrdinalEncoder",
    "PowerTransformer",
    "RobustScaler",
    "SplineTransformer",
    "StandardScaler",
    "add_dummy_feature",
    "PolynomialFeatures",
    "binarize",
    "normalize",
    "scale",
    "robust_scale",
    "maxabs_scale",
    "minmax_scale",
    "label_binarize",
    "quantile_transform",
    "power_transform",
]



import sys
import uuid
import warnings
from collections import defaultdict, deque
from collections.abc import Iterable, Iterator, Sized
from itertools import chain, tee

from ..classes.graph import Graph

__all__ = [
    "is_string_like",
    "iterable",
    "empty_generator",
    "flatten",
    "make_list_of_ints",
    "is_list_of_ints",
    "make_str",
    "generate_unique_node",
    "default_opener",
    "dict_to_numpy_array",
    "dict_to_numpy_array1",
    "dict_to_numpy_array2",
    "is_iterator",
    "arbitrary_element",
    "consume",
    "pairwise",
    "groups",
    "to_tuple",
    "create_random_state",
    "create_py_random_state",
    "PythonRandomInterface",
    "nodes_equal",
    "edges_equal",
    "graphs_equal",
]

# some cookbook stuff
# used in deciding whether something is a bunch of nodes, edges, etc.
# see G.add_nodes and others in Graph Class in networkx/base.py

def is_string_like(obj): ...
def iterable(obj): ...
def empty_generator(): ...
def flatten(obj, result=None): ...
def make_list_of_ints(sequence): ...
def is_list_of_ints(intlist): ...
def make_str(x): ...
def generate_unique_node(): ...
def default_opener(filename: str): ...
def dict_to_numpy_array(d, mapping=None): ...
def dict_to_numpy_array2(d, mapping=None): ...
def dict_to_numpy_array1(d, mapping=None): ...
def is_iterator(obj): ...
def arbitrary_element(iterable): ...

# Recipe from the itertools documentation.
def consume(iterator): ...

# Recipe from the itertools documentation.
def pairwise(iterable, cyclic=False): ...
def groups(many_to_one): ...
def to_tuple(x): ...
def create_random_state(random_state=None): ...

class PythonRandomInterface:
    def __init__(self, rng=None): ...
    def random(self): ...
    def uniform(self, a, b): ...
    def randrange(self, a, b=None): ...

    # NOTE: the numpy implementations of `choice` don't support strings, so
    # this cannot be replaced with self._rng.choice
    def choice(self, seq): ...
    def gauss(self, mu, sigma): ...
    def shuffle(self, seq): ...

    #    Some methods don't match API for numpy RandomState.
    #    Commented out versions are not used by NetworkX

    def sample(self, seq, k): ...
    def randint(self, a, b): ...

    #    exponential as expovariate with 1/argument,
    def expovariate(self, scale): ...

    #    pareto as paretovariate with 1/argument,
    def paretovariate(self, shape): ...

#    weibull as weibullvariate multiplied by beta,
#    def weibullvariate(self, alpha, beta):
#        return self._rng.weibull(alpha) * beta
#
#    def triangular(self, low, high, mode):
#        return self._rng.triangular(low, mode, high)
#
#    def choices(self, seq, weights=None, cum_weights=None, k=1):
#        return self._rng.choice(seq

def create_py_random_state(random_state=None): ...
def nodes_equal(nodes1, nodes2) -> bool: ...
def edges_equal(edges1, edges2) -> bool: ...
def graphs_equal(graph1, graph2) -> bool: ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.debugger import Pdb

PTK3 = ...
_use_simple_prompt = ...
class TerminalPdb(Pdb):
    """Standalone IPython debugger."""
    def __init__(self, *args, pt_session_options=..., **kwargs) -> None:
        ...
    
    def pt_init(self, pt_session_options=...): # -> None:
        """Initialize the prompt session and the prompt loop
        and store them in self.pt_app and self.pt_loop.

        Additional keyword arguments for the PromptSession class
        can be specified in pt_session_options.
        """
        ...
    
    def cmdloop(self, intro=...): # -> None:
        """Repeatedly issue a prompt, accept input, parse an initial prefix
        off the received input, and dispatch to action methods, passing them
        the remainder of the line as argument.

        override the same methods from cmd.Cmd to provide prompt toolkit replacement.
        """
        ...
    
    def do_interact(self, arg): # -> None:
        ...
    


def set_trace(frame=...): # -> None:
    """
    Start debugging from `frame`.

    If frame is not specified, debugging starts from caller's frame.
    """
    ...

if __name__ == '__main__':
    old_trace_dispatch = ...



from setuptools.command.build_ext import (
    customize_compiler as customize_compiler,
    new_compiler as new_compiler,
)

import os
import sys
import glob
import tempfile
import textwrap
import subprocess


def compile_test_program(code, extra_preargs: list = [], extra_postargs: list = []):
    ...


def basic_check_build():
    ...



from typing import ClassVar, Literal, TypeVar
from .covariance._shrunk_covariance import OAS
from scipy import linalg
from .base import TransformerMixin, ClassifierMixin, ClassNamePrefixFeaturesOutMixin
from .base import BaseEstimator
from .utils.multiclass import (
    unique_labels as unique_labels,
    check_classification_targets as check_classification_targets,
)
from .utils.validation import check_is_fitted as check_is_fitted
from .preprocessing import StandardScaler as StandardScaler
from .utils._array_api import get_namespace as get_namespace
from numbers import Real as Real, Integral as Integral
from .utils.extmath import softmax as softmax
from numpy import ndarray
from .covariance import (
    ledoit_wolf as ledoit_wolf,
    empirical_covariance as empirical_covariance,
    shrunk_covariance as shrunk_covariance,
)
from .utils._param_validation import (
    StrOptions as StrOptions,
    Interval as Interval,
    HasMethods as HasMethods,
)
from ._typing import ArrayLike, Int, Float, MatrixLike
from .linear_model._base import LinearClassifierMixin

LinearDiscriminantAnalysis_Self = TypeVar(
    "LinearDiscriminantAnalysis_Self", bound="LinearDiscriminantAnalysis"
)
QuadraticDiscriminantAnalysis_Self = TypeVar(
    "QuadraticDiscriminantAnalysis_Self", bound="QuadraticDiscriminantAnalysis"
)


# Authors: Clemens Brunner
#          Martin Billinger
#          Matthieu Perrot
#          Mathieu Blondel

# License: BSD 3-Clause

import warnings
import numpy as np
import scipy.linalg


__all__ = ["LinearDiscriminantAnalysis", "QuadraticDiscriminantAnalysis"]


class LinearDiscriminantAnalysis(
    ClassNamePrefixFeaturesOutMixin,
    LinearClassifierMixin,
    TransformerMixin,
    BaseEstimator,
):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ArrayLike = ...
    xbar_: ArrayLike = ...
    scalings_: ArrayLike = ...
    priors_: ArrayLike = ...
    means_: ArrayLike = ...
    explained_variance_ratio_: ndarray = ...
    covariance_: ArrayLike = ...
    intercept_: ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        solver: Literal["svd", "lsqr", "eigen", "svd"] = "svd",
        shrinkage: float | None | str = None,
        priors: None | ArrayLike = None,
        n_components: None | Int = None,
        store_covariance: bool = False,
        tol: Float = 1e-4,
        covariance_estimator: None | BaseEstimator | OAS = None,
    ) -> None:
        ...

    def fit(
        self: LinearDiscriminantAnalysis_Self, X: MatrixLike, y: ArrayLike
    ) -> LinearDiscriminantAnalysis_Self:
        ...

    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...


class QuadraticDiscriminantAnalysis(ClassifierMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    scalings_: list[ndarray] = ...
    rotations_: list[ndarray] = ...
    priors_: ArrayLike = ...
    means_: ArrayLike = ...
    covariance_: list[ndarray] = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        *,
        priors: None | ArrayLike = None,
        reg_param: Float = 0.0,
        store_covariance: bool = False,
        tol: Float = 1.0e-4
    ) -> None:
        ...

    def fit(
        self: QuadraticDiscriminantAnalysis_Self, X: MatrixLike, y: ArrayLike
    ) -> QuadraticDiscriminantAnalysis_Self:
        ...

    def decision_function(self, X: MatrixLike) -> ndarray:
        ...

    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def predict_proba(self, X: MatrixLike) -> ndarray:
        ...

    def predict_log_proba(self, X: MatrixLike) -> ndarray:
        ...



from collections.abc import (
    Callable,
    Iterable,
    Iterator,
    Sequence,
)
from typing import (
    Any,
    Generic,
    Literal,
    NamedTuple,
    overload,
)

from matplotlib.axes import (
    Axes as PlotAxes,
    SubplotBase as AxesSubplot,
)
from pandas.core.frame import DataFrame
from pandas.core.generic import NDFrame
from pandas.core.groupby.groupby import (  # , get_groupby as get_groupby
    GroupBy as GroupBy,
)
from pandas.core.groupby.grouper import Grouper
from pandas.core.series import Series
from typing_extensions import TypeAlias

from pandas._typing import (
    S1,
    AggFuncTypeBase,
    AggFuncTypeFrame,
    Axis,
    ByT,
    Level,
    ListLike,
    RandomState,
    Scalar,
)

AggScalar: TypeAlias = str | Callable[..., Any]
ScalarResult = ...

class NamedAgg(NamedTuple):
    column: str = ...
    aggfunc: AggScalar = ...

def generate_property(name: str, klass: type[NDFrame]): ...

class SeriesGroupBy(GroupBy, Generic[S1, ByT]):
    def any(self, skipna: bool = ...) -> Series[bool]: ...
    def all(self, skipna: bool = ...) -> Series[bool]: ...
    def apply(self, func, *args, **kwargs) -> Series: ...
    @overload
    def aggregate(self, func: list[AggFuncTypeBase], *args, **kwargs) -> DataFrame: ...
    @overload
    def aggregate(self, func: AggFuncTypeBase, *args, **kwargs) -> Series: ...
    agg = aggregate
    def transform(self, func: Callable | str, *args, **kwargs) -> Series: ...
    def filter(self, func, dropna: bool = ..., *args, **kwargs): ...
    def nunique(self, dropna: bool = ...) -> Series: ...
    def describe(self, **kwargs) -> DataFrame: ...
    @overload
    def value_counts(
        self,
        normalize: Literal[False] = ...,
        sort: bool = ...,
        ascending: bool = ...,
        bins=...,
        dropna: bool = ...,
    ) -> Series[int]: ...
    @overload
    def value_counts(
        self,
        normalize: Literal[True],
        sort: bool = ...,
        ascending: bool = ...,
        bins=...,
        dropna: bool = ...,
    ) -> Series[float]: ...
    def count(self) -> Series[int]: ...
    def pct_change(
        self,
        periods: int = ...,
        fill_method: str = ...,
        limit=...,
        freq=...,
        axis: Axis = ...,
    ) -> Series[float]: ...
    # Overrides and others from original pylance stubs
    @property
    def is_monotonic_increasing(self) -> bool: ...
    @property
    def is_monotonic_decreasing(self) -> bool: ...
    def bfill(self, limit: int | None = ...) -> Series[S1]: ...
    def cummax(self, axis: Axis = ..., **kwargs) -> Series[S1]: ...
    def cummin(self, axis: Axis = ..., **kwargs) -> Series[S1]: ...
    def cumprod(self, axis: Axis = ..., **kwargs) -> Series[S1]: ...
    def cumsum(self, axis: Axis = ..., **kwargs) -> Series[S1]: ...
    def ffill(self, limit: int | None = ...) -> Series[S1]: ...
    def first(self, **kwargs) -> Series[S1]: ...
    def head(self, n: int = ...) -> Series[S1]: ...
    def last(self, **kwargs) -> Series[S1]: ...
    def max(self, **kwargs) -> Series[S1]: ...
    def mean(self, **kwargs) -> Series[S1]: ...
    def median(self, **kwargs) -> Series[S1]: ...
    def min(self, **kwargs) -> Series[S1]: ...
    def nlargest(self, n: int = ..., keep: str = ...) -> Series[S1]: ...
    def nsmallest(self, n: int = ..., keep: str = ...) -> Series[S1]: ...
    def nth(self, n: int | Sequence[int], dropna: str | None = ...) -> Series[S1]: ...
    def sum(
        self,
        numeric_only: bool = ...,
        min_count: int = ...,
        engine=...,
        engine_kwargs=...,
    ) -> Series[S1]: ...
    def prod(self, numeric_only: bool = ..., min_count: int = ...) -> Series[S1]: ...
    def sem(self, ddof: int = ..., numeric_only: bool = ...) -> Series[float]: ...
    def std(self, ddof: int = ..., numeric_only: bool = ...) -> Series[float]: ...
    def var(self, ddof: int = ..., numeric_only: bool = ...) -> Series[float]: ...
    def tail(self, n: int = ...) -> Series[S1]: ...
    def unique(self) -> Series: ...
    def hist(
        self,
        by=...,
        ax: PlotAxes | None = ...,
        grid: bool = ...,
        xlabelsize: int | None = ...,
        xrot: float | None = ...,
        ylabelsize: int | None = ...,
        yrot: float | None = ...,
        figsize: tuple[float, float] | None = ...,
        bins: int | Sequence = ...,
        backend: str | None = ...,
        legend: bool = ...,
        **kwargs,
    ) -> AxesSubplot: ...
    def idxmax(self, axis: Axis = ..., skipna: bool = ...) -> Series: ...
    def idxmin(self, axis: Axis = ..., skipna: bool = ...) -> Series: ...
    def __iter__(self) -> Iterator[tuple[ByT, Series[S1]]]: ...
    def diff(self, periods: int = ..., axis: Axis = ...) -> Series: ...

class DataFrameGroupBy(GroupBy, Generic[ByT]):
    def any(self, skipna: bool = ...) -> DataFrame: ...
    def all(self, skipna: bool = ...) -> DataFrame: ...
    # error: Overload 3 for "apply" will never be used because its parameters overlap overload 1
    @overload
    def apply(  # type: ignore[misc]
        self, func: Callable[[DataFrame], Scalar | list | dict], *args, **kwargs
    ) -> Series: ...
    @overload
    def apply(
        self, func: Callable[[DataFrame], Series | DataFrame], *args, **kwargs
    ) -> DataFrame: ...
    @overload
    def apply(  # pyright: ignore[reportOverlappingOverload]
        self, func: Callable[[Iterable], float], *args, **kwargs
    ) -> DataFrame: ...
    # error: overload 1 overlaps overload 2 because of different return types
    @overload
    def aggregate(self, arg: Literal["size"]) -> Series: ...  # type: ignore[misc]  # pyright: ignore[reportOverlappingOverload]
    @overload
    def aggregate(self, arg: AggFuncTypeFrame = ..., *args, **kwargs) -> DataFrame:
        """
Aggregate using one or more operations over the specified axis.

Parameters
----------
func : function, str, list, dict or None
    Function to use for aggregating the data. If a function, must either
    work when passed a DataFrame or when passed to DataFrame.apply.

    Accepted combinations are:

    - function
    - string function name
    - list of functions and/or function names, e.g. ``[np.sum, 'mean']``
    - dict of axis labels -> functions, function names or list of such.
    - None, in which case ``**kwargs`` are used with Named Aggregation. Here the
      output has one column for each element in ``**kwargs``. The name of the
      column is keyword, whereas the value determines the aggregation used to compute
      the values in the column.

    Can also accept a Numba JIT function with
    ``engine='numba'`` specified. Only passing a single function is supported
    with this engine.

    If the ``'numba'`` engine is chosen, the function must be
    a user defined function with ``values`` and ``index`` as the
    first and second arguments respectively in the function signature.
    Each group's index will be passed to the user defined function
    and optionally available for use.

    .. versionchanged:: 1.1.0
*args
    Positional arguments to pass to func.
engine : str, default None
    * ``'cython'`` : Runs the function through C-extensions from cython.
    * ``'numba'`` : Runs the function through JIT compiled code from numba.
    * ``None`` : Defaults to ``'cython'`` or globally setting ``compute.use_numba``

    .. versionadded:: 1.1.0
engine_kwargs : dict, default None
    * For ``'cython'`` engine, there are no accepted ``engine_kwargs``
    * For ``'numba'`` engine, the engine can accept ``nopython``, ``nogil``
      and ``parallel`` dictionary keys. The values must either be ``True`` or
      ``False``. The default ``engine_kwargs`` for the ``'numba'`` engine is
      ``{'nopython': True, 'nogil': False, 'parallel': False}`` and will be
      applied to the function

    .. versionadded:: 1.1.0
**kwargs
    * If ``func`` is None, ``**kwargs`` are used to define the output names and
      aggregations via Named Aggregation. See ``func`` entry.
    * Otherwise, keyword arguments to be passed into func.

Returns
-------
DataFrame

See Also
--------
DataFrame.groupby.apply : Apply function func group-wise
    and combine the results together.
DataFrame.groupby.transform : Transforms the Series on each group
    based on the given function.
DataFrame.aggregate : Aggregate using one or more
    operations over the specified axis.

Notes
-----
When using ``engine='numba'``, there will be no "fall back" behavior internally.
The group data and group index will be passed as numpy arrays to the JITed
user defined function, and no alternative execution attempts will be tried.

Functions that mutate the passed object can produce unexpected
behavior or errors and are not supported. See :ref:`gotchas.udf-mutation`
for more details.

.. versionchanged:: 1.3.0

    The resulting dtype will reflect the return value of the passed ``func``,
    see the examples below.

Examples
--------
>>> df = pd.DataFrame(
...     {
...         "A": [1, 1, 2, 2],
...         "B": [1, 2, 3, 4],
...         "C": [0.362838, 0.227877, 1.267767, -0.562860],
...     }
... )

>>> df
   A  B         C
0  1  1  0.362838
1  1  2  0.227877
2  2  3  1.267767
3  2  4 -0.562860

The aggregation is for each column.

>>> df.groupby('A').agg('min')
   B         C
A
1  1  0.227877
2  3 -0.562860

Multiple aggregations

>>> df.groupby('A').agg(['min', 'max'])
    B             C
  min max       min       max
A
1   1   2  0.227877  0.362838
2   3   4 -0.562860  1.267767

Select a column for aggregation

>>> df.groupby('A').B.agg(['min', 'max'])
   min  max
A
1    1    2
2    3    4

User-defined function for aggregation

>>> df.groupby('A').agg(lambda x: sum(x) + 2)
    B          C
A
1       5       2.590715
2       9       2.704907

Different aggregations per column

>>> df.groupby('A').agg({'B': ['min', 'max'], 'C': 'sum'})
    B             C
  min max       sum
A
1   1   2  0.590715
2   3   4  0.704907

To control the output names with different aggregations per column,
pandas supports "named aggregation"

>>> df.groupby("A").agg(
...     b_min=pd.NamedAgg(column="B", aggfunc="min"),
...     c_sum=pd.NamedAgg(column="C", aggfunc="sum"))
   b_min     c_sum
A
1      1  0.590715
2      3  0.704907

- The keywords are the *output* column names
- The values are tuples whose first element is the column to select
  and the second element is the aggregation to apply to that column.
  Pandas provides the ``pandas.NamedAgg`` namedtuple with the fields
  ``['column', 'aggfunc']`` to make it clearer what the arguments are.
  As usual, the aggregation can be a callable or a string alias.

See :ref:`groupby.aggregate.named` for more.

.. versionchanged:: 1.3.0

    The resulting dtype will reflect the return value of the aggregating function.

>>> df.groupby("A")[["B"]].agg(lambda x: x.astype(float).min())
      B
A
1   1.0
2   3.0
        """
        pass
    agg = aggregate
    def transform(self, func: Callable | str, *args, **kwargs) -> DataFrame: ...
    def filter(
        self, func: Callable, dropna: bool = ..., *args, **kwargs
    ) -> DataFrame: ...
    def nunique(self, dropna: bool = ...) -> DataFrame: ...
    @overload
    def __getitem__(self, item: str) -> SeriesGroupBy[Any, ByT]: ...
    @overload
    def __getitem__(self, item: list[str]) -> DataFrameGroupBy[ByT]: ...
    def count(self) -> DataFrame: ...
    def boxplot(
        self,
        grouped: DataFrame,
        subplots: bool = ...,
        column: str | Sequence | None = ...,
        fontsize: float | str = ...,
        rot: float = ...,
        grid: bool = ...,
        ax: PlotAxes | None = ...,
        figsize: tuple[float, float] | None = ...,
        layout: tuple[int, int] | None = ...,
        sharex: bool = ...,
        sharey: bool = ...,
        bins: int | Sequence = ...,
        backend: str | None = ...,
        **kwargs,
    ) -> AxesSubplot | Sequence[AxesSubplot]:
        """
Make box plots from DataFrameGroupBy data.

Parameters
----------
grouped : Grouped DataFrame
subplots : bool
    * ``False`` - no subplots will be used
    * ``True`` - create a subplot for each group.

column : column name or list of names, or vector
    Can be any valid input to groupby.
fontsize : float or str
rot : label rotation angle
grid : Setting this to True will show the grid
ax : Matplotlib axis object, default None
figsize : A tuple (width, height) in inches
layout : tuple (optional)
    The layout of the plot: (rows, columns).
sharex : bool, default False
    Whether x-axes will be shared among subplots.
sharey : bool, default True
    Whether y-axes will be shared among subplots.
backend : str, default None
    Backend to use instead of the backend specified in the option
    ``plotting.backend``. For instance, 'matplotlib'. Alternatively, to
    specify the ``plotting.backend`` for the whole session, set
    ``pd.options.plotting.backend``.
**kwargs
    All other plotting keyword arguments to be passed to
    matplotlib's boxplot function.

Returns
-------
dict of key/value = group key/DataFrame.boxplot return value
or DataFrame.boxplot return value in case subplots=figures=False

Examples
--------
You can create boxplots for grouped data and show them as separate subplots:

.. plot::
    :context: close-figs

    >>> import itertools
    >>> tuples = [t for t in itertools.product(range(1000), range(4))]
    >>> index = pd.MultiIndex.from_tuples(tuples, names=['lvl0', 'lvl1'])
    >>> data = np.random.randn(len(index),4)
    >>> df = pd.DataFrame(data, columns=list('ABCD'), index=index)
    >>> grouped = df.groupby(level='lvl1')
    >>> grouped.boxplot(rot=45, fontsize=12, figsize=(8,10))  # doctest: +SKIP

The ``subplots=False`` option shows the boxplots in a single figure.

.. plot::
    :context: close-figs

    >>> grouped.boxplot(subplots=False, rot=45, fontsize=12)  # doctest: +SKIP
        """
        pass
    # Overrides and others from original pylance stubs
    # These are "properties" but properties can't have all these arguments?!
    def corr(self, method: str | Callable, min_periods: int = ...) -> DataFrame: ...
    def cov(self, min_periods: int = ...) -> DataFrame: ...
    def diff(self, periods: int = ..., axis: Axis = ...) -> DataFrame: ...
    def bfill(self, limit: int | None = ...) -> DataFrame: ...
    def corrwith(
        self,
        other: DataFrame,
        axis: Axis = ...,
        drop: bool = ...,
        method: str = ...,
    ) -> Series: ...
    def cummax(
        self, axis: Axis = ..., numeric_only: bool = ..., **kwargs
    ) -> DataFrame: ...
    def cummin(
        self, axis: Axis = ..., numeric_only: bool = ..., **kwargs
    ) -> DataFrame: ...
    def cumprod(self, axis: Axis = ..., **kwargs) -> DataFrame: ...
    def cumsum(self, axis: Axis = ..., **kwargs) -> DataFrame: ...
    def describe(self, **kwargs) -> DataFrame: ...
    def ffill(self, limit: int | None = ...) -> DataFrame: ...
    def fillna(
        self,
        value,
        method: str | None = ...,
        axis: Axis = ...,
        inplace: Literal[False] = ...,
        limit: int | None = ...,
        downcast: dict | None = ...,
    ) -> DataFrame:
        """
Fill NA/NaN values using the specified method within groups.

Parameters
----------
value : scalar, dict, Series, or DataFrame
    Value to use to fill holes (e.g. 0), alternately a
    dict/Series/DataFrame of values specifying which value to use for
    each index (for a Series) or column (for a DataFrame).  Values not
    in the dict/Series/DataFrame will not be filled. This value cannot
    be a list. Users wanting to use the ``value`` argument and not ``method``
    should prefer :meth:`.DataFrame.fillna` as this
    will produce the same result and be more performant.
method : {{'bfill', 'ffill', None}}, default None
    Method to use for filling holes. ``'ffill'`` will propagate
    the last valid observation forward within a group.
    ``'bfill'`` will use next valid observation to fill the gap.
axis : {0 or 'index', 1 or 'columns'}
    Axis along which to fill missing values. When the :class:`DataFrameGroupBy`
    ``axis`` argument is ``0``, using ``axis=1`` here will produce
    the same results as :meth:`.DataFrame.fillna`. When the
    :class:`DataFrameGroupBy` ``axis`` argument is ``1``, using ``axis=0``
    or ``axis=1`` here will produce the same results.
inplace : bool, default False
    Broken. Do not set to True.
limit : int, default None
    If method is specified, this is the maximum number of consecutive
    NaN values to forward/backward fill within a group. In other words,
    if there is a gap with more than this number of consecutive NaNs,
    it will only be partially filled. If method is not specified, this is the
    maximum number of entries along the entire axis where NaNs will be
    filled. Must be greater than 0 if not None.
downcast : dict, default is None
    A dict of item->dtype of what to downcast if possible,
    or the string 'infer' which will try to downcast to an appropriate
    equal type (e.g. float64 to int64 if possible).

Returns
-------
DataFrame
    Object with missing values filled.

See Also
--------
ffill : Forward fill values within a group.
bfill : Backward fill values within a group.

Examples
--------
>>> df = pd.DataFrame(
...     {
...         "key": [0, 0, 1, 1, 1],
...         "A": [np.nan, 2, np.nan, 3, np.nan],
...         "B": [2, 3, np.nan, np.nan, np.nan],
...         "C": [np.nan, np.nan, 2, np.nan, np.nan],
...     }
... )
>>> df
   key    A    B   C
0    0  NaN  2.0 NaN
1    0  2.0  3.0 NaN
2    1  NaN  NaN 2.0
3    1  3.0  NaN NaN
4    1  NaN  NaN NaN

Propagate non-null values forward or backward within each group along columns.

>>> df.groupby("key").fillna(method="ffill")
     A    B   C
0  NaN  2.0 NaN
1  2.0  3.0 NaN
2  NaN  NaN 2.0
3  3.0  NaN 2.0
4  3.0  NaN 2.0

>>> df.groupby("key").fillna(method="bfill")
     A    B   C
0  2.0  2.0 NaN
1  2.0  3.0 NaN
2  3.0  NaN 2.0
3  3.0  NaN NaN
4  NaN  NaN NaN

Propagate non-null values forward or backward within each group along rows.

>>> df.groupby([0, 0, 1, 1], axis=1).fillna(method="ffill")
   key    A    B    C
0  0.0  0.0  2.0  2.0
1  0.0  2.0  3.0  3.0
2  1.0  1.0  NaN  2.0
3  1.0  3.0  NaN  NaN
4  1.0  1.0  NaN  NaN

>>> df.groupby([0, 0, 1, 1], axis=1).fillna(method="bfill")
   key    A    B    C
0  0.0  NaN  2.0  NaN
1  0.0  2.0  3.0  NaN
2  1.0  NaN  2.0  2.0
3  1.0  3.0  NaN  NaN
4  1.0  NaN  NaN  NaN

Only replace the first NaN element within a group along rows.

>>> df.groupby("key").fillna(method="ffill", limit=1)
     A    B    C
0  NaN  2.0  NaN
1  2.0  3.0  NaN
2  NaN  NaN  2.0
3  3.0  NaN  2.0
4  3.0  NaN  NaN
        """
        pass
    def first(self, **kwargs) -> DataFrame: ...
    def head(self, n: int = ...) -> DataFrame: ...
    def hist(
        self,
        data: DataFrame,
        column: str | Sequence | None = ...,
        by=...,
        grid: bool = ...,
        xlabelsize: int | None = ...,
        xrot: float | None = ...,
        ylabelsize: int | None = ...,
        yrot: float | None = ...,
        ax: PlotAxes | None = ...,
        sharex: bool = ...,
        sharey: bool = ...,
        figsize: tuple[float, float] | None = ...,
        layout: tuple[int, int] | None = ...,
        bins: int | Sequence = ...,
        backend: str | None = ...,
        **kwargs,
    ) -> AxesSubplot | Sequence[AxesSubplot]:
        """
Make a histogram of the DataFrame's columns.

A `histogram`_ is a representation of the distribution of data.
This function calls :meth:`matplotlib.pyplot.hist`, on each series in
the DataFrame, resulting in one histogram per column.

.. _histogram: https://en.wikipedia.org/wiki/Histogram

Parameters
----------
data : DataFrame
    The pandas object holding the data.
column : str or sequence, optional
    If passed, will be used to limit data to a subset of columns.
by : object, optional
    If passed, then used to form histograms for separate groups.
grid : bool, default True
    Whether to show axis grid lines.
xlabelsize : int, default None
    If specified changes the x-axis label size.
xrot : float, default None
    Rotation of x axis labels. For example, a value of 90 displays the
    x labels rotated 90 degrees clockwise.
ylabelsize : int, default None
    If specified changes the y-axis label size.
yrot : float, default None
    Rotation of y axis labels. For example, a value of 90 displays the
    y labels rotated 90 degrees clockwise.
ax : Matplotlib axes object, default None
    The axes to plot the histogram on.
sharex : bool, default True if ax is None else False
    In case subplots=True, share x axis and set some x axis labels to
    invisible; defaults to True if ax is None otherwise False if an ax
    is passed in.
    Note that passing in both an ax and sharex=True will alter all x axis
    labels for all subplots in a figure.
sharey : bool, default False
    In case subplots=True, share y axis and set some y axis labels to
    invisible.
figsize : tuple, optional
    The size in inches of the figure to create. Uses the value in
    `matplotlib.rcParams` by default.
layout : tuple, optional
    Tuple of (rows, columns) for the layout of the histograms.
bins : int or sequence, default 10
    Number of histogram bins to be used. If an integer is given, bins + 1
    bin edges are calculated and returned. If bins is a sequence, gives
    bin edges, including left edge of first bin and right edge of last
    bin. In this case, bins is returned unmodified.

backend : str, default None
    Backend to use instead of the backend specified in the option
    ``plotting.backend``. For instance, 'matplotlib'. Alternatively, to
    specify the ``plotting.backend`` for the whole session, set
    ``pd.options.plotting.backend``.

legend : bool, default False
    Whether to show the legend.

    .. versionadded:: 1.1.0

**kwargs
    All other plotting keyword arguments to be passed to
    :meth:`matplotlib.pyplot.hist`.

Returns
-------
matplotlib.AxesSubplot or numpy.ndarray of them

See Also
--------
matplotlib.pyplot.hist : Plot a histogram using matplotlib.

Examples
--------
This example draws a histogram based on the length and width of
some animals, displayed in three bins

.. plot::
    :context: close-figs

    >>> df = pd.DataFrame({
    ...     'length': [1.5, 0.5, 1.2, 0.9, 3],
    ...     'width': [0.7, 0.2, 0.15, 0.2, 1.1]
    ...     }, index=['pig', 'rabbit', 'duck', 'chicken', 'horse'])
    >>> hist = df.hist(bins=3)
        """
        pass
    def idxmax(
        self, axis: Axis = ..., skipna: bool = ..., numeric_only: bool = ...
    ) -> DataFrame:
        """
Return index of first occurrence of maximum over requested axis.

NA/null values are excluded.

Parameters
----------
axis : {{0 or 'index', 1 or 'columns'}}, default None
    The axis to use. 0 or 'index' for row-wise, 1 or 'columns' for column-wise.
    If axis is not provided, grouper's axis is used.

    .. versionchanged:: 2.0.0

skipna : bool, default True
    Exclude NA/null values. If an entire row/column is NA, the result
    will be NA.
numeric_only : bool, default False
    Include only `float`, `int` or `boolean` data.

    .. versionadded:: 1.5.0

Returns
-------
Series
    Indexes of maxima along the specified axis.

Raises
------
ValueError
    * If the row/column is empty

See Also
--------
Series.idxmax : Return index of the maximum element.

Notes
-----
This method is the DataFrame version of ``ndarray.argmax``.

Examples
--------
Consider a dataset containing food consumption in Argentina.

>>> df = pd.DataFrame({'consumption': [10.51, 103.11, 55.48],
...                    'co2_emissions': [37.2, 19.66, 1712]},
...                    index=['Pork', 'Wheat Products', 'Beef'])

>>> df
                consumption  co2_emissions
Pork                  10.51         37.20
Wheat Products       103.11         19.66
Beef                  55.48       1712.00

By default, it returns the index for the maximum value in each column.

>>> df.idxmax()
consumption     Wheat Products
co2_emissions             Beef
dtype: object

To return the index for the maximum value in each row, use ``axis="columns"``.

>>> df.idxmax(axis="columns")
Pork              co2_emissions
Wheat Products     consumption
Beef              co2_emissions
dtype: object
        """
        pass
    def idxmin(
        self, axis: Axis = ..., skipna: bool = ..., numeric_only: bool = ...
    ) -> DataFrame:
        """
Return index of first occurrence of minimum over requested axis.

NA/null values are excluded.

Parameters
----------
axis : {{0 or 'index', 1 or 'columns'}}, default None
    The axis to use. 0 or 'index' for row-wise, 1 or 'columns' for column-wise.
    If axis is not provided, grouper's axis is used.

    .. versionchanged:: 2.0.0

skipna : bool, default True
    Exclude NA/null values. If an entire row/column is NA, the result
    will be NA.
numeric_only : bool, default False
    Include only `float`, `int` or `boolean` data.

    .. versionadded:: 1.5.0

Returns
-------
Series
    Indexes of minima along the specified axis.

Raises
------
ValueError
    * If the row/column is empty

See Also
--------
Series.idxmin : Return index of the minimum element.

Notes
-----
This method is the DataFrame version of ``ndarray.argmin``.

Examples
--------
Consider a dataset containing food consumption in Argentina.

>>> df = pd.DataFrame({'consumption': [10.51, 103.11, 55.48],
...                    'co2_emissions': [37.2, 19.66, 1712]},
...                    index=['Pork', 'Wheat Products', 'Beef'])

>>> df
                consumption  co2_emissions
Pork                  10.51         37.20
Wheat Products       103.11         19.66
Beef                  55.48       1712.00

By default, it returns the index for the minimum value in each column.

>>> df.idxmin()
consumption                Pork
co2_emissions    Wheat Products
dtype: object

To return the index for the minimum value in each row, use ``axis="columns"``.

>>> df.idxmin(axis="columns")
Pork                consumption
Wheat Products    co2_emissions
Beef                consumption
dtype: object
        """
        pass
    def last(self, **kwargs) -> DataFrame: ...
    def max(self, **kwargs) -> DataFrame: ...
    def mean(self, **kwargs) -> DataFrame: ...
    def median(self, **kwargs) -> DataFrame: ...
    def min(self, **kwargs) -> DataFrame: ...
    def nth(self, n: int | Sequence[int], dropna: str | None = ...) -> DataFrame: ...
    def pct_change(
        self,
        periods: int = ...,
        fill_method: str = ...,
        limit=...,
        freq=...,
        axis: Axis = ...,
    ) -> DataFrame: ...
    def prod(self, numeric_only: bool = ..., min_count: int = ...) -> DataFrame: ...
    def quantile(
        self, q: float = ..., interpolation: str = ..., numeric_only: bool = ...
    ) -> DataFrame: ...
    def resample(self, rule, *args, **kwargs) -> Grouper: ...
    def sample(
        self,
        n: int | None = ...,
        frac: float | None = ...,
        replace: bool = ...,
        weights: ListLike | None = ...,
        random_state: RandomState | None = ...,
    ) -> DataFrame: ...
    def sem(self, ddof: int = ..., numeric_only: bool = ...) -> DataFrame: ...
    def shift(
        self,
        periods: int = ...,
        freq: str = ...,
        axis: Axis = ...,
        fill_value=...,
    ) -> DataFrame: ...
    @overload
    def skew(
        self,
        axis: Axis = ...,
        skipna: bool = ...,
        numeric_only: bool = ...,
        *,
        level: Level,
        **kwargs,
    ) -> DataFrame:
        """
Return unbiased skew within groups.

Normalized by N-1.

Parameters
----------
axis : {0 or 'index', 1 or 'columns', None}, default 0
    Axis for the function to be applied on.

    Specifying ``axis=None`` will apply the aggregation across both axes.

    .. versionadded:: 2.0.0

skipna : bool, default True
    Exclude NA/null values when computing the result.

numeric_only : bool, default False
    Include only float, int, boolean columns.

**kwargs
    Additional keyword arguments to be passed to the function.

Returns
-------
DataFrame

See Also
--------
DataFrame.skew : Return unbiased skew over requested axis.

Examples
--------
>>> arrays = [['falcon', 'parrot', 'cockatoo', 'kiwi',
...            'lion', 'monkey', 'rabbit'],
...           ['bird', 'bird', 'bird', 'bird',
...            'mammal', 'mammal', 'mammal']]
>>> index = pd.MultiIndex.from_arrays(arrays, names=('name', 'class'))
>>> df = pd.DataFrame({'max_speed': [389.0, 24.0, 70.0, np.nan,
...                                  80.5, 21.5, 15.0]},
...                   index=index)
>>> df
                max_speed
name     class
falcon   bird        389.0
parrot   bird         24.0
cockatoo bird         70.0
kiwi     bird          NaN
lion     mammal       80.5
monkey   mammal       21.5
rabbit   mammal       15.0
>>> gb = df.groupby(["class"])
>>> gb.skew()
        max_speed
class
bird     1.628296
mammal   1.669046
>>> gb.skew(skipna=False)
        max_speed
class
bird          NaN
mammal   1.669046
        """
        pass
    @overload
    def skew(
        self,
        axis: Axis = ...,
        skipna: bool = ...,
        level: None = ...,
        numeric_only: bool = ...,
        **kwargs,
    ) -> Series: ...
    def std(self, ddof: int = ..., numeric_only: bool = ...) -> DataFrame: ...
    def sum(
        self,
        numeric_only: bool = ...,
        min_count: int = ...,
        engine=...,
        engine_kwargs=...,
    ) -> DataFrame: ...
    def tail(self, n: int = ...) -> DataFrame: ...
    def take(self, indices: Sequence, axis: Axis = ..., **kwargs) -> DataFrame:
        """
Return the elements in the given *positional* indices in each group.

This means that we are not indexing according to actual values in
the index attribute of the object. We are indexing according to the
actual position of the element in the object.

If a requested index does not exist for some group, this method will raise.
To get similar behavior that ignores indices that don't exist, see
:meth:`.DataFrameGroupBy.nth`.

Parameters
----------
indices : array-like
    An array of ints indicating which positions to take.
axis : {0 or 'index', 1 or 'columns', None}, default 0
    The axis on which to select elements. ``0`` means that we are
    selecting rows, ``1`` means that we are selecting columns.
**kwargs
    For compatibility with :meth:`numpy.take`. Has no effect on the
    output.

Returns
-------
DataFrame
    An DataFrame containing the elements taken from each group.

See Also
--------
DataFrame.take : Take elements from a Series along an axis.
DataFrame.loc : Select a subset of a DataFrame by labels.
DataFrame.iloc : Select a subset of a DataFrame by positions.
numpy.take : Take elements from an array along an axis.

Examples
--------
>>> df = pd.DataFrame([('falcon', 'bird', 389.0),
...                    ('parrot', 'bird', 24.0),
...                    ('lion', 'mammal', 80.5),
...                    ('monkey', 'mammal', np.nan),
...                    ('rabbit', 'mammal', 15.0)],
...                   columns=['name', 'class', 'max_speed'],
...                   index=[4, 3, 2, 1, 0])
>>> df
     name   class  max_speed
4  falcon    bird      389.0
3  parrot    bird       24.0
2    lion  mammal       80.5
1  monkey  mammal        NaN
0  rabbit  mammal       15.0
>>> gb = df.groupby([1, 1, 2, 2, 2])

Take elements at positions 0 and 1 along the axis 0 (default).

Note how the indices selected in the result do not correspond to
our input indices 0 and 1. That's because we are selecting the 0th
and 1st rows, not rows whose indices equal 0 and 1.

>>> gb.take([0, 1])
       name   class  max_speed
1 4  falcon    bird      389.0
  3  parrot    bird       24.0
2 2    lion  mammal       80.5
  1  monkey  mammal        NaN

The order of the specified indices influences the order in the result.
Here, the order is swapped from the previous example.

>>> gb.take([1, 0])
       name   class  max_speed
1 3  parrot    bird       24.0
  4  falcon    bird      389.0
2 1  monkey  mammal        NaN
  2    lion  mammal       80.5

Take elements at indices 1 and 2 along the axis 1 (column selection).

We may take elements using negative integers for positive indices,
starting from the end of the object, just like with Python lists.

>>> gb.take([-1, -2])
       name   class  max_speed
1 3  parrot    bird       24.0
  4  falcon    bird      389.0
2 0  rabbit  mammal       15.0
  1  monkey  mammal        NaN
        """
        pass
    def tshift(self, periods: int, freq=..., axis: Axis = ...) -> DataFrame: ...
    def var(self, ddof: int = ..., numeric_only: bool = ...) -> DataFrame: ...
    @overload
    def value_counts(
        self,
        subset: ListLike | None = ...,
        normalize: Literal[False] = ...,
        sort: bool = ...,
        ascending: bool = ...,
        dropna: bool = ...,
    ) -> Series[int]: ...
    @overload
    def value_counts(
        self,
        subset: ListLike | None,
        normalize: Literal[True],
        sort: bool = ...,
        ascending: bool = ...,
        dropna: bool = ...,
    ) -> Series[float]: ...
    def __getattr__(self, name: str) -> SeriesGroupBy[Any, ByT]: ...
    def __iter__(self) -> Iterator[tuple[ByT, DataFrame]]: ...



from vispy.util.svg.color import Color
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from .line import LineVisual
from .markers import MarkersVisual
from .visual import CompoundVisual

class LinePlotVisual(CompoundVisual):

    _line_kwargs = ...
    _marker_kwargs = ...
    _valid_kwargs = ...
    _kw_trans = ...

    def __init__(
        self,
        data: ArrayLike | None = None,
        color: Color | str = "k",
        symbol: str | None = None,
        line_kind: str = "-",
        width: float = 1.0,
        marker_size: float = 10.0,
        edge_color: Color | str = "k",
        face_color: Color | str = "w",
        edge_width: float = 1.0,
        connect: str | ArrayLike = "strip",
    ): ...
    def set_data(self, data: ArrayLike | None = None, **kwargs): ...



from typing import Any, ClassVar, Literal, TypeVar
from ..utils.sparsefuncs import mean_variance_axis as mean_variance_axis
from numpy.random import RandomState
from scipy.sparse.linalg import svds as svds
from numpy import ndarray
from ..utils.extmath import (
    randomized_svd as randomized_svd,
    safe_sparse_dot as safe_sparse_dot,
    svd_flip as svd_flip,
)
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from .._typing import Int, Float, MatrixLike, ArrayLike
from ..utils import check_array as check_array, check_random_state as check_random_state
from ..utils.validation import check_is_fitted as check_is_fitted

TruncatedSVD_Self = TypeVar("TruncatedSVD_Self", bound="TruncatedSVD")

import numpy as np
import scipy.sparse as sp

__all__ = ["TruncatedSVD"]


class TruncatedSVD(ClassNamePrefixFeaturesOutMixin, TransformerMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    singular_values_: ndarray = ...
    explained_variance_ratio_: ndarray = ...
    explained_variance_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: Int = 2,
        *,
        algorithm: Literal["arpack", "randomized", "randomized"] = "randomized",
        n_iter: Int = 5,
        n_oversamples: Int = 10,
        power_iteration_normalizer: Literal[
            "auto", "QR", "LU", "none", "auto"
        ] = "auto",
        random_state: RandomState | None | Int = None,
        tol: Float = 0.0,
    ) -> None:
        ...

    def fit(
        self: TruncatedSVD_Self, X: MatrixLike | ArrayLike, y: Any = None
    ) -> TruncatedSVD_Self:
        ...

    def fit_transform(self, X: MatrixLike | ArrayLike, y: Any = None) -> ndarray:
        ...

    def transform(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...



from typing import Any, Callable, Optional, TypeVar, overload

_C = TypeVar("_C", bound=Callable[..., Any])

@overload
def staff_member_required(
    view_func: _C,
    redirect_field_name: Optional[str] = ...,
    login_url: str = ...,
) -> _C: ...
@overload
def staff_member_required(
    view_func: None = ...,
    redirect_field_name: Optional[str] = ...,
    login_url: str = ...,
) -> Callable[..., Any]: ...




import numpy as np
from typing import Any
from ._splitter import Splitter

from numpy import float32 as DTYPE
from numpy import float64 as DOUBLE


TREE_LEAF: int


class Node:
    left_child: int
    right_child: int
    feature: int
    threshold: float
    impurity: float
    n_node_samples: int
    weighted_n_node_samples: float


class DepthFirstTreeBuilder(TreeBuilder):
    """Build a decision tree in depth-first fashion."""

    def __init__(self, splitter: Splitter, min_samples_split: int,
                  min_samples_leaf: int,  min_weight_leaf: float,
                  max_depth: int,
                  min_impurity_decrease: float) -> None: ...

    def build(
        self,
        tree: Tree,
        X: object,
        y: np.ndarray,
        sample_weight: np.ndarray|None=None,
    ) -> None: ...


class BestFirstTreeBuilder(TreeBuilder):
    """Build a decision tree in best-first fashion.
    The best node to expand is given by the node at the frontier that has the
    highest impurity improvement.
    """
    max_leaf_nodes: int

    def __init__(self, splitter: Splitter, min_samples_split: int,
                  min_samples_leaf: int,  min_weight_leaf: float,
                  max_depth: int, max_leaf_nodes: int,
                  min_impurity_decrease: float) -> None: ...

    def build(
        self,
        tree: Tree,
        X: object,
        y: np.ndarray,
        sample_weight: np.ndarray|None=None,
    ) -> None: ...

 

class Tree:

    n_features: int
    n_classes: int
    n_outputs: int
    max_n_classes: int

    max_dept: int
    node_count: int
    capacity: int
    nodes: np.ndarray
    value: np.ndarray
    value_stride: int

    def predict(self, X) -> np.ndarray: ...
    def apply(self, X) -> np.ndarray: ...
    def decision_path(self, X) -> Any: ...
    def compute_node_depths(self): ...
    def compute_feature_importances(self, normalize): ...


class TreeBuilder:

    splitter: Splitter
    min_samples_split: int
    min_samples_leaf: int
    min_weight_leaf: float
    max_depth: int
    min_impurity_decrease: float

    def build(
        self,
        tree: Tree,
        X,
        y,
        sample_weight,
    ): ...

    
def ccp_pruning_path(orig_tree: Tree) -> dict: ...
    



from typing import Any, Callable, TypeVar

from django.middleware.csrf import CsrfViewMiddleware

_F = TypeVar("_F", bound=Callable[..., Any])

def csrf_protect(__view: _F) -> _F: ...

class _EnsureCsrfToken(CsrfViewMiddleware): ...

def requires_csrf_token(__view: _F) -> _F: ...

class _EnsureCsrfCookie(CsrfViewMiddleware):
    get_response: None
    def process_view(
        self, request: Any, callback: Any, callback_args: Any, callback_kwargs: Any
    ) -> Any: ...

def ensure_csrf_cookie(__view: _F) -> _F: ...
def csrf_exempt(view_func: _F) -> _F: ...



import datetime
from .units import ConversionInterface
import numpy as np
from dateutil import rrule
from .ticker import Formatter, Locator

UTC: datetime.timezone = ...
EPOCH_OFFSET: float = ...
JULIAN_OFFSET: float = ...
MICROSECONDLY: int = ...
HOURS_PER_DAY: int = ...
MIN_PER_HOUR: int = ...
SEC_PER_MIN: int = ...
MONTHS_PER_YEAR: int = ...
DAYS_PER_WEEK: int = ...
DAYS_PER_MONTH: int = ...
DAYS_PER_YEAR: int = ...
MINUTES_PER_DAY: int = ...
SEC_PER_HOUR: int = ...
SEC_PER_DAY: int = ...
SEC_PER_WEEK: int = ...
MUSECONDS_PER_DAY: int = ...
MO: rrule.weekday = ...
TU: rrule.weekday = ...
WE: rrule.weekday = ...
TH: rrule.weekday = ...
FR: rrule.weekday = ...
SA: rrule.weekday = ...
SU: rrule.weekday = ...
WEEKDAYS: tuple[rrule.weekday] = ...
YEARLY: int = ...
MONTHLY: int = ...
WEEKLY: int = ...
DAILY: int = ...
HOURLY: int = ...
MINUTELY: int = ...
SECONDLY: int = ...

def set_epoch(epoch: str): ...
def get_epoch() -> str: ...
def datestr2num(d: str, default: datetime.datetime = ...): ...
def date2num(d: datetime.datetime | np.datetime64) -> float: ...
def julian2num(j: float) -> float: ...
def num2julian(n: float) -> float: ...
def num2date(x: float, tz: str | datetime.tzinfo = ...) -> datetime.datetime: ...
def num2timedelta(x) -> tuple[datetime.timedelta, list[datetime.timedelta]]: ...
def drange(
    dstart: datetime.datetime | datetime.date,
    dend: datetime.datetime | datetime.date,
    delta: datetime.timedelta,
) -> np.ndarray: ...

class DateFormatter(Formatter):
    def __init__(
        self, fmt: str, tz: str | datetime.tzinfo = ..., *, usetex: bool = ...
    ) -> None: ...
    def __call__(self, x, pos=...): ...
    def set_tzinfo(self, tz: str | datetime.tzinfo): ...

class ConciseDateFormatter(Formatter):

    formats: list[str] = ...
    offset_formats: list[str] = ...
    zero_formats: list[str] = ...

    def __init__(
        self,
        locator: Locator,
        tz: str | datetime.tzinfo = ...,
        formats: list[str] = ...,
        offset_formats: list[str] = ...,
        zero_formats: list[str] = ...,
        show_offset: bool = ...,
        *,
        usetex: bool = ...
    ) -> None: ...
    def __call__(self, x, pos=...): ...
    def format_ticks(self, values)-> None: ...
    def get_offset(self): ...
    def format_data_short(self, value): ...

class AutoDateFormatter(Formatter):
    def __init__(
        self,
        locator: Locator,
        tz: str | datetime.tzinfo = ...,
        defaultfmt: str = ...,
        *,
        usetex: bool = ...
    ) -> None: ...
    def __call__(self, x, pos=...): ...

class rrulewrapper:
    def __init__(
        self, freq: int, tzinfo: datetime.tzinfo | None = ..., **kwargs
    ) -> None: ...
    def set(self, **kwargs)-> None: ...
    def __getattr__(self, name): ...
    def __setstate__(self, state)-> None: ...

class DateLocator(Locator):

    hms0d = ...
    def __init__(self, tz: str | datetime.tzinfo = ...) -> None: ...
    def set_tzinfo(self, tz: str | datetime.tzinfo)-> None: ...
    def datalim_to_dt(self): ...
    def viewlim_to_dt(self): ...
    def nonsingular(self, vmin, vmax): ...

class RRuleLocator(DateLocator):
    def __init__(self, o, tz: str | datetime.tzinfo = ...) -> None: ...
    def __call__(self): ...
    def tick_values(self, vmin, vmax): ...
    @staticmethod
    def get_unit_generic(freq): ...

class AutoDateLocator(DateLocator):
    def __init__(
        self,
        tz: str | datetime.tzinfo = ...,
        minticks: int = ...,
        maxticks: int = ...,
        interval_multiples: bool = True,
    ) -> None: ...
    def __call__(self): ...
    def tick_values(self, vmin, vmax): ...
    def nonsingular(self, vmin, vmax): ...
    def get_locator(self, dmin, dmax): ...

class YearLocator(RRuleLocator):
    def __init__(
        self, base=..., month=..., day=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class MonthLocator(RRuleLocator):
    def __init__(
        self, bymonth=..., bymonthday=..., interval=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class WeekdayLocator(RRuleLocator):
    def __init__(
        self, byweekday=..., interval=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class DayLocator(RRuleLocator):
    def __init__(
        self, bymonthday=..., interval=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class HourLocator(RRuleLocator):
    def __init__(
        self, byhour=..., interval=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class MinuteLocator(RRuleLocator):
    def __init__(
        self, byminute=..., interval=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class SecondLocator(RRuleLocator):
    def __init__(
        self, bysecond=..., interval=..., tz: str | datetime.tzinfo = ...
    ) -> None: ...

class MicrosecondLocator(DateLocator):
    def __init__(self, interval=..., tz: str | datetime.tzinfo = ...) -> None: ...
    def set_axis(self, axis)-> None: ...
    def set_view_interval(self, vmin: float, vmax: float)-> None: ...
    def set_data_interval(self, vmin: float, vmax: float)-> None: ...
    def __call__(self): ...
    def tick_values(self, vmin, vmax): ...

def epoch2num(e: list[float]) -> np.ndarray: ...
def num2epoch(d: list[float]) -> np.ndarray: ...
def date_ticker_factory(span, tz: str | datetime.tzinfo = ..., numticks=...): ...

class DateConverter(ConversionInterface):
    def __init__(self, *, interval_multiples=...) -> None: ...
    def axisinfo(self, unit: None | datetime.tzinfo, axis): ...
    @staticmethod
    def convert(value, unit, axis): ...
    @staticmethod
    def default_units(x, axis): ...

class ConciseDateConverter(DateConverter):
    def __init__(
        self,
        formats=...,
        zero_formats=...,
        offset_formats=...,
        show_offset=...,
        *,
        interval_multiples=...
    ) -> None: ...
    def axisinfo(self, unit, axis): ...

class _SwitchableDateConverter:
    def axisinfo(self, *args, **kwargs): ...
    def default_units(self, *args, **kwargs): ...
    def convert(self, *args, **kwargs): ...

__all__ = (
    "datestr2num",
    "date2num",
    "num2date",
    "num2timedelta",
    "drange",
    "epoch2num",
    "num2epoch",
    "set_epoch",
    "get_epoch",
    "DateFormatter",
    "ConciseDateFormatter",
    "AutoDateFormatter",
    "DateLocator",
    "RRuleLocator",
    "AutoDateLocator",
    "YearLocator",
    "MonthLocator",
    "WeekdayLocator",
    "DayLocator",
    "HourLocator",
    "MinuteLocator",
    "SecondLocator",
    "MicrosecondLocator",
    "rrule",
    "MO",
    "TU",
    "WE",
    "TH",
    "FR",
    "SA",
    "SU",
    "YEARLY",
    "MONTHLY",
    "WEEKLY",
    "DAILY",
    "HOURLY",
    "MINUTELY",
    "SECONDLY",
    "MICROSECONDLY",
    "relativedelta",
    "DateConverter",
    "ConciseDateConverter",
    "rrulewrapper",
)



from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union

from django.db.migrations.graph import MigrationGraph
from django.db.migrations.migration import Migration
from django.db.migrations.operations.base import Operation
from django.db.migrations.questioner import MigrationQuestioner
from django.db.migrations.state import ProjectState
from django.db.models.fields import Field

class MigrationAutodetector:
    from_state: ProjectState = ...
    to_state: ProjectState = ...
    questioner: MigrationQuestioner = ...
    existing_apps: Set[Any] = ...
    def __init__(
        self,
        from_state: ProjectState,
        to_state: ProjectState,
        questioner: Optional[MigrationQuestioner] = ...,
    ) -> None: ...
    def changes(
        self,
        graph: MigrationGraph,
        trim_to_apps: Optional[Set[str]] = ...,
        convert_apps: Optional[Set[str]] = ...,
        migration_name: Optional[str] = ...,
    ) -> Dict[str, List[Migration]]: ...
    def deep_deconstruct(self, obj: Any) -> Any: ...
    def only_relation_agnostic_fields(
        self, fields: List[Tuple[str, Field[Any, Any]]]
    ) -> List[
        Tuple[str, List[Any], Dict[str, Union[Callable[..., Any], int, str]]]
    ]: ...
    def check_dependency(
        self,
        operation: Operation,
        dependency: Tuple[str, str, Optional[str], Union[bool, str]],
    ) -> bool: ...
    def add_operation(
        self,
        app_label: str,
        operation: Operation,
        dependencies: Optional[
            List[Tuple[str, str, Optional[str], Union[bool, str]]]
        ] = ...,
        beginning: bool = ...,
    ) -> None: ...
    def swappable_first_key(self, item: Tuple[str, str]) -> Tuple[str, str]: ...
    renamed_models: Any = ...
    renamed_models_rel: Any = ...
    def generate_renamed_models(self) -> None: ...
    def generate_created_models(self) -> None: ...
    def generate_created_proxies(self) -> None: ...
    def generate_deleted_models(self) -> None: ...
    def generate_deleted_proxies(self) -> None: ...
    renamed_fields: Any = ...
    def generate_renamed_fields(self) -> None: ...
    def generate_added_fields(self) -> None: ...
    def generate_removed_fields(self) -> None: ...
    def generate_altered_fields(self) -> None: ...
    def create_altered_indexes(self) -> None: ...
    def generate_added_indexes(self) -> None: ...
    def generate_removed_indexes(self) -> None: ...
    def generate_altered_unique_together(self) -> None: ...
    def generate_altered_index_together(self) -> None: ...
    def generate_altered_db_table(self) -> None: ...
    def generate_altered_options(self) -> None: ...
    def generate_altered_order_with_respect_to(self) -> None: ...
    def generate_altered_managers(self) -> None: ...
    def arrange_for_graph(
        self,
        changes: Dict[str, List[Migration]],
        graph: MigrationGraph,
        migration_name: Optional[str] = ...,
    ) -> Dict[str, List[Migration]]: ...
    @classmethod
    def suggest_name(cls, ops: List[Operation]) -> str: ...
    @classmethod
    def parse_number(cls, name: str) -> int: ...



from typing import IO
from vispy._typing import FileLike

# -*- coding: utf-8 -*-
# Copyright (c) 2015 Michael Dawson-Haggerty
# Distributed under the MIT License.
# Copied from the trimesh project.
# See https://github.com/mikedh/trimesh for more information.
# See https://github.com/mikedh/trimesh/blob/master/LICENSE.md for
# the license.

import numpy as np

class HeaderError(Exception):
    pass

# define a numpy datatype for the data section of a binary STL file
_stl_dtype = ...
# define a numpy datatype for the header of a binary STL file
_stl_dtype_header = ...

def load_stl(file_obj: FileLike, file_type=None): ...
def load_stl_binary(file_obj: FileLike): ...
def load_stl_ascii(file_obj: FileLike): ...

_stl_loaders: dict = ...



from typing import Mapping

from collections import Counter
from itertools import chain, combinations
from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = [
    "triangles",
    "average_clustering",
    "clustering",
    "transitivity",
    "square_clustering",
    "generalized_degree",
]

def triangles(G: Graph, nodes=None) -> Mapping: ...
def average_clustering(
    G: Graph, nodes=None, weight=None, count_zeros: bool = True
) -> float: ...
def clustering(G: Graph, nodes=None, weight=None) -> float | dict: ...
def transitivity(G: Graph) -> float: ...
def square_clustering(G: Graph, nodes=None) -> Mapping: ...
def generalized_degree(G: Graph, nodes=None): ...



from os import makedirs as makedirs, remove as remove
from os.path import exists as exists
from ._base import RemoteFileMetadata as RemoteFileMetadata
from numpy import ndarray
from io import BytesIO as BytesIO
from ..utils._bunch import Bunch
from . import get_data_home as get_data_home

import logging
import numpy as np

import joblib

# The original data can be found at:
# https://biodiversityinformatics.amnh.org/open_source/maxent/samples.zip
SAMPLES = ...

# The original data can be found at:
# https://biodiversityinformatics.amnh.org/open_source/maxent/coverages.zip
COVERAGES = ...

DATA_ARCHIVE_NAME: str = ...


logger = ...


def construct_grids(batch) -> ndarray:
    ...


def fetch_species_distributions(
    *, data_home: None | str = None, download_if_missing: bool = True
) -> Bunch:
    ...



from typing import Any, Type
from matplotlib._typing import *
from matplotlib.text import Text
from matplotlib.font_manager import FontProperties
from matplotlib.transforms import Affine2DBase, Transform
from matplotlib.backend_bases import (
    FigureCanvasBase,
    FigureManagerBase,
    GraphicsContextBase,
    RendererBase,
    _Backend,
)
from encodings.utf_8 import StreamWriter
from io import BytesIO, TextIOWrapper

backend_version = ...

def escape_cdata(s)-> str: ...
def escape_comment(s)-> str: ...
def escape_attrib(s)-> str: ...
def short_float_fmt(x)-> str: ...

class XMLWriter:
    def __init__(self, file: TextIOWrapper | StreamWriter) -> None: ...
    def start(self, tag: str, attrib: dict[str, str] = ..., **extra) -> int: ...
    def comment(self, comment: str) -> None: ...
    def data(self, text: str) -> None: ...
    def end(self, tag: str | None = ..., indent: bool = ...) -> None: ...
    def close(self, id: int) -> None: ...
    def element(
        self, tag: str, text: str | None = ..., attrib: dict[str, str] = ..., **extra
    ) -> None: ...
    def flush(self) -> None: ...

def generate_transform(transform_list=...)-> str: ...
def generate_css(attrib=...)-> str: ...

class RendererSVG(RendererBase):
    def __init__(
        self, width, height, svgwriter, basename=..., image_dpi=..., *, metadata=...
    ) -> None: ...
    def finalize(self) -> None: ...
    def open_group(self, s: str, gid: str | None = ...) -> None: ...
    def close_group(self, s: str) -> None: ...
    def option_image_nocomposite(self) -> bool: ...
    def draw_path(self, gc, path, transform, rgbFace=...)-> None: ...
    def draw_markers(
        self,
        gc: GraphicsContextBase,
        marker_path,
        marker_trans: Transform,
        path,
        trans: Transform,
        rgbFace=...,
    )-> None: ...
    def draw_path_collection(
        self,
        gc,
        master_transform,
        paths,
        all_transforms,
        offsets,
        offsetTrans,
        facecolors,
        edgecolors,
        linewidths,
        linestyles,
        antialiaseds,
        urls,
        offset_position,
    )-> None: ...
    def draw_gouraud_triangle(self, gc: GraphicsContextBase, points, colors, trans)-> None: ...
    def draw_gouraud_triangles(
        self, gc, triangles_array, colors_array, transform: Transform
    )-> None: ...
    def option_scale_image(self) -> bool: ...
    def get_image_magnification(self) -> float: ...
    def draw_image(
        self,
        gc: GraphicsContextBase,
        x: Scalar,
        y: Scalar,
        im,
        transform: Affine2DBase = ...,
    )-> None: ...
    def draw_tex(self, gc, x, y, s, prop, angle, *, mtext=...)-> None: ...
    def draw_text(
        self,
        gc: GraphicsContextBase,
        x: float,
        y: float,
        s: str,
        prop: FontProperties,
        angle: float,
        ismath=...,
        mtext: Text = ...,
    )-> None: ...
    def flipy(self) -> bool: ...
    def get_canvas_width_height(self)-> tuple[float, float]: ...
    def get_text_width_height_descent(
        self, s: str, prop: FontProperties, ismath: bool
    ) -> tuple[float, float, float]: ...

class FigureCanvasSVG(FigureCanvasBase):
    filetypes: dict[str, str] = ...
    fixed_dpi: float = ...

    def print_svg(
        self,
        filename: BytesIO | str,
        *args,
        bbox_inches_restore=...,
        metadata: dict[str, Any] = ...
    ) -> None: ...
    def print_svgz(self, filename: str | PathLike | FileLike, *args, **kwargs)-> None: ...
    def get_default_filetype(self)-> str: ...
    def draw(self)-> None: ...

FigureManagerSVG: Type[FigureManagerBase] = ...
svgProlog: str = ...

class _BackendSVG(_Backend):
    FigureCanvas = FigureCanvasSVG



from .._typing import MatrixLike, ArrayLike, Float
from scipy.special import expit as logistic_sigmoid, xlogy as xlogy

# Author: Issam H. Laradji <issam.laradji@gmail.com>
# License: BSD 3 clause

import numpy as np


def inplace_identity(X: MatrixLike) -> None:
    ...


def inplace_logistic(X: MatrixLike) -> None:
    ...


def inplace_tanh(X: MatrixLike):
    ...


def inplace_relu(X: MatrixLike) -> None:
    ...


def inplace_softmax(X: MatrixLike) -> None:
    ...


ACTIVATIONS: dict = ...


def inplace_identity_derivative(Z: MatrixLike, delta: MatrixLike):
    ...


def inplace_logistic_derivative(Z: MatrixLike, delta: MatrixLike):
    ...


def inplace_tanh_derivative(Z: MatrixLike, delta: MatrixLike):
    ...


def inplace_relu_derivative(Z: MatrixLike, delta: MatrixLike) -> None:
    ...


DERIVATIVES: dict = ...


def squared_loss(
    y_true: MatrixLike | ArrayLike, y_pred: MatrixLike | ArrayLike
) -> Float:
    ...


def log_loss(y_true: MatrixLike | ArrayLike, y_prob: MatrixLike) -> Float:
    ...


def binary_log_loss(y_true: MatrixLike | ArrayLike, y_prob: MatrixLike) -> Float:
    ...


LOSS_FUNCTIONS: dict = ...



from itertools import count

from ...classes.graph import Graph
from .community_utils import is_partition
from ...utils import BinaryHeap, not_implemented_for, py_random_state

__all__ = ["kernighan_lin_bisection"]

@py_random_state(4)
def kernighan_lin_bisection(
    G: Graph,
    partition: tuple | None = None,
    max_iter: int = 10,
    weight="weight",
    seed=None,
) -> tuple: ...



"""
This type stub file was generated by pyright.
"""

from IPython.core import magic_arguments
from IPython.core.magic import Magics, cell_magic, line_magic, magics_class

"""Magic functions for running cells in various scripts."""
def script_args(f):
    """single decorator for adding script args"""
    ...

@magics_class
class ScriptMagics(Magics):
    """Magics for talking to scripts
    
    This defines a base `%%script` cell magic for running a cell
    with a program in a subprocess, and registers a few top-level
    magics that call %%script with common interpreters.
    """
    event_loop = ...
    script_magics = ...
    script_paths = ...
    def __init__(self, shell=...) -> None:
        ...
    
    def __del__(self): # -> None:
        ...
    
    @magic_arguments.magic_arguments()
    @script_args
    @cell_magic("script")
    def shebang(self, line, cell):
        """Run a cell via a shell command

        The `%%script` line is like the #! line of script,
        specifying a program (bash, perl, ruby, etc.) with which to run.

        The rest of the cell is run by that program.

        Examples
        --------
        ::

            In [1]: %%script bash
               ...: for i in 1 2 3; do
               ...:   echo $i
               ...: done
            1
            2
            3
        """
        ...
    
    @line_magic("killbgscripts")
    def killbgscripts(self, _nouse_=...): # -> None:
        """Kill all BG processes started by %%script and its family."""
        ...
    
    def kill_bg_processes(self): # -> None:
        """Kill all BG processes which are still running."""
        ...
    





from numpy.typing import ArrayLike
from ..classes.digraph import DiGraph
from ..classes.multigraph import MultiGraph
from ..classes.multidigraph import MultiDiGraph
from ..classes.graph import Graph

import heapq
import math
from itertools import chain, combinations, zip_longest
from operator import itemgetter

from ..classes.graph import Graph
from ..utils import py_random_state, random_weighted_sample

__all__ = [
    "configuration_model",
    "directed_configuration_model",
    "expected_degree_graph",
    "havel_hakimi_graph",
    "directed_havel_hakimi_graph",
    "degree_sequence_tree",
    "random_degree_sequence_graph",
]

chaini = ...

@py_random_state(2)
def configuration_model(deg_sequence, create_using=None, seed=None) -> MultiGraph: ...
@py_random_state(3)
def directed_configuration_model(
    in_degree_sequence, out_degree_sequence, create_using=None, seed=None
) -> MultiDiGraph: ...
@py_random_state(1)
def expected_degree_graph(w: ArrayLike, seed=None, selfloops=True) -> Graph: ...
def havel_hakimi_graph(deg_sequence, create_using=None): ...
def directed_havel_hakimi_graph(
    in_deg_sequence, out_deg_sequence, create_using=None
) -> DiGraph: ...
def degree_sequence_tree(deg_sequence, create_using=None): ...
@py_random_state(1)
def random_degree_sequence_graph(sequence, seed=None, tries: int = 10) -> Graph: ...

class DegreeSequenceRandomGraph:
    # class to generate random graphs with a given degree sequence
    # use random_degree_sequence_graph()
    def __init__(self, degree, rng): ...
    def generate(self): ...
    def update_remaining(self, u, v, aux_graph=None): ...
    def p(self, u, v): ...
    def q(self, u, v): ...
    def suitable_edge(self): ...
    def phase1(self): ...
    def phase2(self): ...
    def phase3(self): ...



from numpy.typing import ArrayLike

from collections import defaultdict, deque

from ...classes.graph import Graph
from ...utils import py_random_state

__all__ = ["louvain_communities", "louvain_partitions"]

@py_random_state("seed")
def louvain_communities(
    G: Graph, weight="weight", resolution=1, threshold=0.0000001, seed=None
) -> ArrayLike: ...
@py_random_state("seed")
def louvain_partitions(
    G: Graph, weight="weight", resolution=1, threshold=0.0000001, seed=None
): ...



from numpy.typing import NDArray
import numpy as np

from .._shared.filters import gaussian

def binary_blobs(
    length: int = 512,
    blob_size_fraction: float = 0.1,
    n_dim: int = 2,
    volume_fraction: float = 0.5,
    seed=None,
) -> NDArray: ...



from numpy.typing import NDArray
from typing import Literal
import numpy as np

from .._shared._geometry import polygon_clip
from .._shared.version_requirements import require

def _ellipse_in_shape(shape, center, radii, rotation=0.0): ...
def ellipse(
    r, c, r_radius, c_radius, shape: tuple | None = None, rotation=0.0
) -> NDArray: ...
def disk(center: tuple, radius, *, shape: tuple | None = None) -> NDArray: ...
@require("matplotlib", ">=3.0.3")
def polygon_perimeter(
    r, c, shape: tuple | None = None, clip: bool = False
) -> NDArray: ...
def set_color(image, coords, color, alpha=1): ...
def line(r0: int, c0: int, r1: int, c1: int): ...
def line_aa(r0: int, c0: int, r1: int, c1: int): ...
def polygon(r, c, shape: tuple | None = None) -> NDArray: ...
def circle_perimeter(
    r: int,
    c: int,
    radius: int,
    method: Literal["bresenham", "andres"] = "bresenham",
    shape: tuple | None = None,
): ...
def circle_perimeter_aa(r: int, c: int, radius: int, shape: tuple | None = None): ...
def ellipse_perimeter(
    r: int,
    c: int,
    r_radius: int,
    c_radius: int,
    orientation=0,
    shape: tuple | None = None,
): ...
def bezier_curve(
    r0: int,
    c0: int,
    r1: int,
    c1: int,
    r2: int,
    c2: int,
    weight,
    shape: tuple | None = None,
): ...
def rectangle(
    start: tuple,
    end: tuple | None = None,
    extent: tuple | None = None,
    shape: tuple | None = None,
): ...
@require("matplotlib", ">=3.0.3")
def rectangle_perimeter(
    start: tuple,
    end: tuple | None = None,
    extent: tuple | None = None,
    shape: tuple | None = None,
    clip: bool = False,
): ...
def _rectangle_slice(start, end=None, extent=None): ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

# Use OSX cocoa/quartz to get glyph bitmaps

import numpy as np
from ctypes import byref, c_int32, c_byte

from ...ext.cocoapy import (
    cf,
    ct,
    quartz,
    CFRange,
    CFSTR,
    CGGlyph,
    UniChar,
    kCTFontFamilyNameAttribute,
    kCTFontBoldTrait,
    kCTFontItalicTrait,
    kCTFontSymbolicTrait,
    kCTFontTraitsAttribute,
    kCTFontAttributeName,
    kCGImageAlphaPremultipliedLast,
    kCFNumberSInt32Type,
    ObjCClass,
)
from ._vispy_fonts import _vispy_fonts, _get_vispy_font_filename

_font_dict: dict = ...

def _load_vispy_font(face, bold, italic): ...
def _load_font(face, bold, italic): ...
def _load_glyph(f, char, glyphs_dict): ...
def _get_k_p_a(font, left, right): ...
def _list_fonts(): ...



"""
This type stub file was generated by pyright.
"""

from IPython.core.displayhook import DisplayHook

"""Terminal input and output prompts."""
class Prompts:
    def __init__(self, shell) -> None:
        ...
    
    def vi_mode(self):
        ...
    
    def in_prompt_tokens(self):
        ...
    
    def continuation_prompt_tokens(self, width=...):
        ...
    
    def rewrite_prompt_tokens(self):
        ...
    
    def out_prompt_tokens(self):
        ...
    


class ClassicPrompts(Prompts):
    def in_prompt_tokens(self):
        ...
    
    def continuation_prompt_tokens(self, width=...):
        ...
    
    def rewrite_prompt_tokens(self):
        ...
    
    def out_prompt_tokens(self):
        ...
    


class RichPromptDisplayHook(DisplayHook):
    """Subclass of base display hook using coloured prompt"""
    def write_output_prompt(self): # -> None:
        ...
    
    def write_format_data(self, format_dict, md_dict=...) -> None:
        ...
    





from numpy import ndarray
from scipy.sparse import issparse as issparse
from ..base import TransformerMixin
from .._typing import MatrixLike, ArrayLike
from ..utils.validation import check_is_fitted as check_is_fitted

# Author: V. Michel, A. Gramfort
# License: BSD 3 clause

import numpy as np

###############################################################################
# Mixin class for feature agglomeration.


class AgglomerationTransform(TransformerMixin):
    def transform(self, X: MatrixLike) -> ndarray:
        ...

    def inverse_transform(self, Xred: MatrixLike | ArrayLike) -> ndarray:
        ...



from vispy.util.svg.color import Color
from vispy.util.event import Event

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

# Code translated from glumpy

import numpy as np

from .widget import Widget
from ...visuals import Visual
from ...gloo import VertexBuffer
from ...color import Color

# Translated from
# http://www.piclist.com/tecHREF/datafile/charset/
#     extractor/charset_extractor.htm
__font_6x8__ = ...

VERTEX_SHADER: str = ...

FRAGMENT_SHADER: str = ...

class Console(Widget):
    def __init__(self, text_color: Color | str = "black", font_size: float = 12.0, **kwargs): ...
    def on_resize(self, event: Event): ...
    def clear(self): ...
    def write(self, text: str = "", wrap: str | bool = True): ...
    @property
    def text_color(self): ...
    @text_color.setter
    def text_color(self, color): ...
    @property
    def font_size(self): ...
    @font_size.setter
    def font_size(self, font_size): ...

class ConsoleVisual(Visual):
    def __init__(self, text_color, font_size, **kwargs): ...
    @property
    def size(self): ...
    @size.setter
    def size(self, s): ...
    @property
    def text_color(self): ...
    @text_color.setter
    def text_color(self, color): ...
    @property
    def font_size(self): ...
    @font_size.setter
    def font_size(self, font_size): ...
    def _resize_buffers(self, font_scale): ...
    def _prepare_draw(self, view): ...
    def _prepare_transforms(self, view): ...
    def clear(self): ...
    def write(self, text: str = "", wrap: str | bool = True): ...
    def _do_pending_writes(self): ...
    def _insert_text_buf(self, line, idx): ...



from typing import Iterable, Literal, Sequence
from numpy.random import RandomState
from scipy import linalg as linalg
from numpy import ndarray
from collections.abc import Iterable
from scipy.sparse import spmatrix
from ..utils.random import sample_without_replacement as sample_without_replacement
from .._typing import Int, ArrayLike, Float, MatrixLike
from ..utils import (
    check_array as check_array,
    check_random_state as check_random_state,
    shuffle as util_shuffle,
)
from ..preprocessing import MultiLabelBinarizer as MultiLabelBinarizer

# Authors: B. Thirion, G. Varoquaux, A. Gramfort, V. Michel, O. Grisel,
#          G. Louppe, J. Nothman
# License: BSD 3 clause

import numbers
import array
import warnings

import numpy as np
import scipy.sparse as sp


def make_classification(
    n_samples: Int = 100,
    n_features: Int = 20,
    *,
    n_informative: Int = 2,
    n_redundant: Int = 2,
    n_repeated: Int = 0,
    n_classes: Int = 2,
    n_clusters_per_class: Int = 2,
    weights: None | ArrayLike = None,
    flip_y: Float = 0.01,
    class_sep: Float = 1.0,
    hypercube: bool = True,
    shift: float | None | ArrayLike = 0.0,
    scale: float | None | ArrayLike = 1.0,
    shuffle: bool = True,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_multilabel_classification(
    n_samples: Int = 100,
    n_features: Int = 20,
    *,
    n_classes: Int = 5,
    n_labels: Int = 2,
    length: Int = 50,
    allow_unlabeled: bool = True,
    sparse: bool = False,
    return_indicator: Literal["dense", "sparse", "dense"] | bool = "dense",
    return_distributions: bool = False,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray | spmatrix, ndarray, ndarray]:
    ...


def make_hastie_10_2(
    n_samples: Int = 12000, *, random_state: RandomState | None | Int = None
) -> tuple[ndarray, ndarray]:
    ...


def make_regression(
    n_samples: Int = 100,
    n_features: Int = 100,
    *,
    n_informative: Int = 10,
    n_targets: Int = 1,
    bias: Float = 0.0,
    effective_rank: None | Int = None,
    tail_strength: Float = 0.5,
    noise: Float = 0.0,
    shuffle: bool = True,
    coef: bool = False,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray, ndarray]:
    ...


def make_circles(
    n_samples: int | tuple[int, int] = 100,
    *,
    shuffle: bool = True,
    noise: None | Float = None,
    random_state: RandomState | None | Int = None,
    factor: Float = 0.8,
) -> tuple[ndarray, ndarray]:
    ...


def make_moons(
    n_samples: int | tuple[int, int] = 100,
    *,
    shuffle: bool = True,
    noise: None | Float = None,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_blobs(
    n_samples: ArrayLike | int = 100,
    n_features: Int = 2,
    *,
    centers: None | MatrixLike | int = None,
    cluster_std: float | Sequence[float] = 1.0,
    center_box: tuple[float, float] = ...,
    shuffle: bool = True,
    random_state: RandomState | None | Int = None,
    return_centers: bool = False,
) -> tuple[ndarray, ndarray, ndarray] | tuple[ndarray, ndarray]:
    ...


def make_friedman1(
    n_samples: Int = 100,
    n_features: Int = 10,
    *,
    noise: Float = 0.0,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_friedman2(
    n_samples: Int = 100,
    *,
    noise: Float = 0.0,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_friedman3(
    n_samples: Int = 100,
    *,
    noise: Float = 0.0,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_low_rank_matrix(
    n_samples: Int = 100,
    n_features: Int = 100,
    *,
    effective_rank: Int = 10,
    tail_strength: Float = 0.5,
    random_state: RandomState | None | Int = None,
) -> ndarray:
    ...


# TODO(1.3): Change argument `data_transposed` default from True to False.
# TODO(1.3): Deprecate data_transposed, always return data not transposed.
def make_sparse_coded_signal(
    n_samples: Int,
    *,
    n_components: Int,
    n_features: Int,
    n_nonzero_coefs: Int,
    random_state: RandomState | None | Int = None,
    data_transposed: str | bool = "warn",
) -> map | tuple[ndarray, ndarray, ndarray]:
    ...


def make_sparse_uncorrelated(
    n_samples: Int = 100,
    n_features: Int = 10,
    *,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_spd_matrix(
    n_dim: Int, *, random_state: RandomState | None | Int = None
) -> ndarray:
    ...


def make_sparse_spd_matrix(
    dim: Int = 1,
    *,
    alpha: Float = 0.95,
    norm_diag: bool = False,
    smallest_coef: Float = 0.1,
    largest_coef: Float = 0.9,
    random_state: RandomState | None | Int = None,
) -> ndarray | spmatrix:
    ...


def make_swiss_roll(
    n_samples: Int = 100,
    *,
    noise: Float = 0.0,
    random_state: RandomState | None | Int = None,
    hole: bool = False,
) -> tuple[ndarray, ndarray]:
    ...


def make_s_curve(
    n_samples: Int = 100,
    *,
    noise: Float = 0.0,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_gaussian_quantiles(
    *,
    mean: None | ArrayLike = None,
    cov: Float = 1.0,
    n_samples: Int = 100,
    n_features: Int = 2,
    n_classes: Int = 3,
    shuffle: bool = True,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray]:
    ...


def make_biclusters(
    shape: Iterable[MatrixLike] | tuple[int, int],
    n_clusters: Int,
    *,
    noise: Float = 0.0,
    minval: Int = 10,
    maxval: Int = 100,
    shuffle: bool = True,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray, ndarray]:
    ...


def make_checkerboard(
    shape: tuple | tuple[int, int],
    n_clusters: MatrixLike | Int | tuple[int, int],
    *,
    noise: Float = 0.0,
    minval: Int = 10,
    maxval: Int = 100,
    shuffle: bool = True,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray, ndarray]:
    ...



from .base import clone as clone
from ._config import (
    get_config as get_config,
    set_config as set_config,
    config_context as config_context,
)
from .utils._show_versions import show_versions as show_versions
import sys as sys
import logging as logging
import os as os
import random as random

logger = ...


# PEP0440 compatible formatted version, see:
# https://www.python.org/dev/peps/pep-0440/
#
# Generic release markers:
#   X.Y.0   # For first release after an increment in Y
#   X.Y.Z   # For bugfix releases
#
# Admissible pre-release markers:
#   X.Y.ZaN   # Alpha release
#   X.Y.ZbN   # Beta release
#   X.Y.ZrcN  # Release Candidate
#   X.Y.Z     # Final release
#
# Dev branch marker is: 'X.Y.dev' or 'X.Y.devN' where N is an integer.
# 'X.Y.dev0' is the canonical version of 'X.Y.dev'
#
__version__: str = ...


def setup_module(module):
    ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from numpy.random import RandomState
from scipy import linalg as linalg
from ..linear_model import (
    Lasso as Lasso,
    orthogonal_mp_gram as orthogonal_mp_gram,
    LassoLars as LassoLars,
    Lars as Lars,
)
from numpy import ndarray
from ..utils._param_validation import (
    Hidden as Hidden,
    Interval as Interval,
    StrOptions as StrOptions,
)
from ..utils.extmath import (
    randomized_svd as randomized_svd,
    row_norms as row_norms,
    svd_flip as svd_flip,
)
from joblib import effective_n_jobs as effective_n_jobs
from numbers import Integral as Integral, Real as Real
from math import ceil as ceil
from ..base import BaseEstimator, TransformerMixin, ClassNamePrefixFeaturesOutMixin
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from .._typing import ArrayLike, MatrixLike, Int, Float
from ..utils import (
    check_array as check_array,
    check_random_state as check_random_state,
    gen_even_slices as gen_even_slices,
    gen_batches as gen_batches,
    deprecated,
)
from ..utils.validation import check_is_fitted as check_is_fitted

DictionaryLearning_Self = TypeVar("DictionaryLearning_Self", bound="DictionaryLearning")
SparseCoder_Self = TypeVar("SparseCoder_Self", bound="SparseCoder")
MiniBatchDictionaryLearning_Self = TypeVar(
    "MiniBatchDictionaryLearning_Self", bound="MiniBatchDictionaryLearning"
)

# Author: Vlad Niculae, Gael Varoquaux, Alexandre Gramfort
# License: BSD 3 clause

import time
import sys
import itertools
import warnings

import numpy as np


# XXX : could be moved to the linear_model module
def sparse_encode(
    X: ArrayLike,
    dictionary: MatrixLike,
    *,
    gram: None | MatrixLike = None,
    cov: None | MatrixLike = None,
    algorithm: Literal[
        "lasso_lars", "lasso_cd", "lars", "omp", "threshold", "lasso_lars"
    ] = "lasso_lars",
    n_nonzero_coefs: None | Int = None,
    alpha: None | Float = None,
    copy_cov: bool = True,
    init: None | MatrixLike = None,
    max_iter: Int = 1000,
    n_jobs: None | Int = None,
    check_input: bool = True,
    verbose: Int = 0,
    positive: bool = False,
) -> ndarray:
    ...


def dict_learning(
    X: ArrayLike,
    n_components: Int,
    *,
    alpha: Int,
    max_iter: Int = 100,
    tol: Float = 1e-8,
    method: Literal["lars", "cd", "lars"] = "lars",
    n_jobs: None | Int = None,
    dict_init: None | MatrixLike = None,
    code_init: None | MatrixLike = None,
    callback: None | Callable = None,
    verbose: bool = False,
    random_state: RandomState | None | Int = None,
    return_n_iter: bool = False,
    positive_dict: bool = False,
    positive_code: bool = False,
    method_max_iter: Int = 1000,
) -> tuple[ndarray, ndarray, ndarray, int]:
    ...


def dict_learning_online(
    X: ArrayLike,
    n_components: None | int = 2,
    *,
    alpha: Float = 1,
    n_iter: str | Int = "deprecated",
    max_iter: None | Int = None,
    return_code: bool = True,
    dict_init: None | MatrixLike = None,
    callback: None | Callable = None,
    batch_size: str | Int = "warn",
    verbose: bool = False,
    shuffle: bool = True,
    n_jobs: None | Int = None,
    method: Literal["lars", "cd", "lars"] = "lars",
    iter_offset: str | Int = "deprecated",
    random_state: RandomState | None | Int = None,
    return_inner_stats: str | bool = "deprecated",
    inner_stats: str | tuple[ArrayLike, ArrayLike] = "deprecated",
    return_n_iter: str | bool = "deprecated",
    positive_dict: bool = False,
    positive_code: bool = False,
    method_max_iter: Int = 1000,
    tol: Float = 1e-3,
    max_no_improvement: Int = 10,
) -> tuple[ndarray, ndarray, int]:
    ...


class _BaseSparseCoding(ClassNamePrefixFeaturesOutMixin, TransformerMixin):
    def __init__(
        self,
        transform_algorithm: str,
        transform_n_nonzero_coefs: None | int,
        transform_alpha: float | None | int,
        split_sign: bool,
        n_jobs,
        positive_code: bool,
        transform_max_iter: int,
    ) -> None:
        ...

    def transform(self, X: ArrayLike) -> ndarray:
        ...


class SparseCoder(_BaseSparseCoding, BaseEstimator):
    feature_names_in_: ndarray = ...

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        dictionary: MatrixLike,
        *,
        transform_algorithm: Literal[
            "lasso_lars", "lasso_cd", "lars", "omp", "threshold", "omp"
        ] = "omp",
        transform_n_nonzero_coefs: None | Int = None,
        transform_alpha: None | Float = None,
        split_sign: bool = False,
        n_jobs: None | Int = None,
        positive_code: bool = False,
        transform_max_iter: Int = 1000,
    ) -> None:
        ...

    def fit(self: SparseCoder_Self, X: Any, y: Any = None) -> SparseCoder_Self:
        ...

    def transform(self, X: ArrayLike, y: Any = None) -> ndarray:
        ...

    @property
    def n_components_(self) -> int:
        ...

    @property
    def n_features_in_(self) -> int:
        ...


class DictionaryLearning(_BaseSparseCoding, BaseEstimator):
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    error_: ndarray = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        alpha: Float = 1,
        max_iter: Int = 1000,
        tol: Float = 1e-8,
        fit_algorithm: Literal["lars", "cd", "lars"] = "lars",
        transform_algorithm: Literal[
            "lasso_lars", "lasso_cd", "lars", "omp", "threshold", "omp"
        ] = "omp",
        transform_n_nonzero_coefs: None | Int = None,
        transform_alpha: None | Float = None,
        n_jobs: None | int = None,
        code_init: None | MatrixLike = None,
        dict_init: None | MatrixLike = None,
        verbose: bool = False,
        split_sign: bool = False,
        random_state: RandomState | None | Int = None,
        positive_code: bool = False,
        positive_dict: bool = False,
        transform_max_iter: Int = 1000,
    ) -> None:
        ...

    def fit(
        self: DictionaryLearning_Self, X: MatrixLike, y: Any = None
    ) -> DictionaryLearning_Self:
        ...


class MiniBatchDictionaryLearning(_BaseSparseCoding, BaseEstimator):
    n_steps_: int = ...
    n_iter_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    components_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_components: None | Int = None,
        *,
        alpha: Float = 1,
        n_iter: str | Int = "deprecated",
        max_iter: None | Int = None,
        fit_algorithm: Literal["lars", "cd", "lars"] = "lars",
        n_jobs: None | Int = None,
        batch_size: str | Int = "warn",
        shuffle: bool = True,
        dict_init: None | MatrixLike = None,
        transform_algorithm: Literal[
            "lasso_lars", "lasso_cd", "lars", "omp", "threshold", "omp"
        ] = "omp",
        transform_n_nonzero_coefs: None | Int = None,
        transform_alpha: None | Float = None,
        verbose: int | bool = False,
        split_sign: bool = False,
        random_state: RandomState | None | Int = None,
        positive_code: bool = False,
        positive_dict: bool = False,
        transform_max_iter: Int = 1000,
        callback: None | Callable = None,
        tol: Float = 1e-3,
        max_no_improvement: Int = 10,
    ) -> None:
        ...

    @deprecated(  # type: ignore
        "The attribute `iter_offset_` is deprecated in 1.1 and will be removed in 1.3."
    )
    @property
    def iter_offset_(self) -> int:
        ...

    @deprecated(  # type: ignore
        "The attribute `random_state_` is deprecated in 1.1 and will be removed in 1.3."
    )
    @property
    def random_state_(self) -> RandomState:
        ...

    @deprecated(  # type: ignore
        "The attribute `inner_stats_` is deprecated in 1.1 and will be removed in 1.3."
    )
    @property
    def inner_stats_(self) -> tuple[ndarray, ndarray]:
        ...

    def fit(
        self: MiniBatchDictionaryLearning_Self, X: MatrixLike, y: Any = None
    ) -> MiniBatchDictionaryLearning_Self:
        ...

    def partial_fit(
        self: MiniBatchDictionaryLearning_Self,
        X: MatrixLike,
        y: Any = None,
        iter_offset: str | Int = "deprecated",
    ) -> MiniBatchDictionaryLearning_Self:
        ...



from numpy.distutils.misc_util import Configuration
from numpy.distutils.core import setup

# Author: Alexandre Gramfort <alexandre.gramfort@inria.fr>
# License: BSD 3 clause
import os

import numpy


def configuration(parent_package: str = "", top_path=None):
    ...



from ..classes.multigraph import MultiGraph
from typing import Mapping
from ..classes.multidigraph import MultiDiGraph
from ..classes.graph import Graph
import warnings
from locale import getpreferredencoding

from ..classes.graph import Graph
from ..utils import open_file

__all__ = [
    "write_dot",
    "read_dot",
    "graphviz_layout",
    "pydot_layout",
    "to_pydot",
    "from_pydot",
]

def write_dot(G: Graph, path): ...
def read_dot(path) -> MultiGraph | MultiDiGraph: ...
def from_pydot(P): ...
def to_pydot(N): ...
def graphviz_layout(G: Graph, prog="neato", root=None): ...
def pydot_layout(G: Graph, prog="neato", root=None) -> Mapping: ...



from numpy.typing import ArrayLike
from functools import partial
from itertools import accumulate, chain

from ...classes.graph import Graph

__all__ = ["join"]

def join(rooted_trees: ArrayLike, label_attribute: str | None = None): ...



from typing import Mapping
from .classes.graph import Graph

__all__ = ["convert_node_labels_to_integers", "relabel_nodes"]

def relabel_nodes(G: Graph, mapping: Mapping, copy=True): ...
def convert_node_labels_to_integers(
    G: Graph, first_label=0, ordering: str = "default", label_attribute=None
): ...



__all__ = ["capacity_scaling"]

from itertools import chain
from math import log

from ...classes.graph import Graph

from ...utils import BinaryHeap, arbitrary_element, not_implemented_for

def capacity_scaling(
    G: Graph,
    demand: str = "demand",
    capacity: str = "capacity",
    weight: str = "weight",
    heap=...,
): ...



from typing import Callable
from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import functools

import numpy as np
from ...util import logger
from functools import wraps

def arg_to_array(func: Callable) -> Callable: ...
def as_vec4(obj: ArrayLike, default: ArrayLike = ...) -> ArrayLike: ...
def arg_to_vec4(func): ...

class TransformCache(object):
    def __init__(self, max_age=1): ...
    def get(self, path): ...
    def _create(self, path): ...
    def roll(self): ...



from typing import Any, Callable, ClassVar, TypeVar
from numpy.random import RandomState
from scipy.stats import chi2 as chi2
from scipy import linalg as linalg
from numpy import ndarray
from ..utils.extmath import fast_logdet as fast_logdet
from ..utils._param_validation import Interval as Interval
from numbers import Integral as Integral, Real as Real
from .._typing import MatrixLike, Int, Float
from ..utils import check_random_state as check_random_state, check_array as check_array
from . import empirical_covariance as empirical_covariance, EmpiricalCovariance

MinCovDet_Self = TypeVar("MinCovDet_Self", bound="MinCovDet")

# Author: Virgile Fritsch <virgile.fritsch@inria.fr>
#
# License: BSD 3 clause

import warnings
import numpy as np


# Minimum Covariance Determinant
#   Implementing of an algorithm by Rousseeuw & Van Driessen described in
#   (A Fast Algorithm for the Minimum Covariance Determinant Estimator,
#   1999, American Statistical Association and the American Society
#   for Quality, TECHNOMETRICS)
# XXX Is this really a public function? It's not listed in the docs or
# exported by sklearn.covariance. Deprecate?
def c_step(
    X: MatrixLike,
    n_support: Int,
    remaining_iterations: Int = 30,
    initial_estimates: None | tuple[Any, Any] = None,
    verbose: bool = False,
    cov_computation_method: Callable = ...,
    random_state: RandomState | None | Int = None,
) -> tuple[ndarray, ndarray, ndarray]:
    ...


def select_candidates(
    X: MatrixLike,
    n_support: Int,
    n_trials: int | tuple[ndarray, ndarray] | tuple[Any, Any],
    select: Int = 1,
    n_iter: Int = 30,
    verbose: bool = False,
    cov_computation_method: Callable = ...,
    random_state: None | Int | RandomState = None,
) -> tuple[ndarray, ndarray, ndarray, ndarray] | tuple[ndarray, ndarray, ndarray]:
    ...


def fast_mcd(
    X: MatrixLike,
    support_fraction: None | Float = None,
    cov_computation_method: Callable = ...,
    random_state: None | Int | RandomState = None,
) -> tuple[ndarray, ndarray, ndarray, ndarray] | tuple[ndarray, ndarray, ndarray]:
    ...


class MinCovDet(EmpiricalCovariance):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    dist_: ndarray = ...
    support_: ndarray = ...
    precision_: ndarray = ...
    covariance_: ndarray = ...
    location_: ndarray = ...
    raw_support_: ndarray = ...
    raw_covariance_: ndarray = ...
    raw_location_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...
    _nonrobust_covariance = ...

    def __init__(
        self,
        *,
        store_precision: bool = True,
        assume_centered: bool = False,
        support_fraction: None | Float = None,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(self: MinCovDet_Self, X: MatrixLike, y: Any = None) -> MinCovDet_Self:
        ...

    def correct_covariance(self, data: MatrixLike) -> ndarray:
        ...

    def reweight_covariance(self, data: MatrixLike) -> tuple[ndarray, ndarray, ndarray]:
        ...



from operator import attrgetter as attrgetter
from abc import ABCMeta, abstractmethod as abstractmethod
from numpy import ndarray
from ..base import TransformerMixin
from scipy.sparse import issparse as issparse, csc_matrix as csc_matrix
from scipy.sparse._csr import csr_matrix
from .._typing import MatrixLike, ArrayLike
from ..utils import (
    check_array as check_array,
    safe_mask as safe_mask,
    safe_sqr as safe_sqr,
)

# Authors: G. Varoquaux, A. Gramfort, L. Buitinck, J. Nothman
# License: BSD 3 clause

import warnings

import numpy as np


class SelectorMixin(TransformerMixin, metaclass=ABCMeta):
    def get_support(self, indices: bool = False) -> ndarray:
        ...

    def transform(self, X: MatrixLike) -> csr_matrix | ndarray:
        ...

    def inverse_transform(self, X: MatrixLike) -> ndarray:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...



from typing import Literal
from ._sag_fast import sag32 as sag32, sag64 as sag64
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from ._base import make_dataset as make_dataset
from ..utils.extmath import row_norms as row_norms
from numpy import ndarray
from numpy.random.mtrand import RandomState
from .._typing import Float, Int, MatrixLike, ArrayLike
from ..utils import check_array as check_array

# Authors: Tom Dupre la Tour <tom.dupre-la-tour@m4x.org>
#
# License: BSD 3 clause

import warnings

import numpy as np


def get_auto_step_size(
    max_squared_sum: Float,
    alpha_scaled: Float,
    loss: Literal["log", "squared", "multinomial"],
    fit_intercept: bool,
    n_samples: None | Int = None,
    is_saga: bool = False,
) -> Float:
    ...


def sag_solver(
    X: MatrixLike | ArrayLike,
    y: ArrayLike,
    sample_weight: None | ArrayLike = None,
    loss: Literal["log", "log", "squared", "multinomial"] = "log",
    alpha: Float = 1.0,
    beta: Float = 0.0,
    max_iter: Int = 1000,
    tol: Float = 0.001,
    verbose: Int = 0,
    random_state: None | Int | RandomState = None,
    check_input: bool = True,
    max_squared_sum: None | Float = None,
    warm_start_mem: dict[str, ndarray] | None | dict = None,
    is_saga: bool = False,
) -> tuple[ndarray, int, dict] | tuple[ndarray, int, dict[str, ndarray | int]]:
    ...



from typing import ClassVar, Literal, Sequence, TypeVar
from scipy import sparse as sparse
from itertools import chain as chain
from ..base import BaseEstimator
from ..preprocessing import FunctionTransformer as FunctionTransformer
from scipy.sparse import spmatrix
from ..utils._bunch import Bunch
from ..utils.parallel import delayed as delayed, Parallel as Parallel
from ..utils.validation import (
    check_array as check_array,
    check_is_fitted as check_is_fitted,
)
from collections import Counter as Counter
from numpy import ndarray, dtype
from ..base import clone as clone, TransformerMixin
from ..utils import check_pandas_support as check_pandas_support
from ..utils.metaestimators import _BaseComposition
from .._typing import Float, Int, ArrayLike, MatrixLike

ColumnTransformer_Self = TypeVar("ColumnTransformer_Self", bound="ColumnTransformer")


import numpy as np


__all__ = ["ColumnTransformer", "make_column_transformer", "make_column_selector"]


_ERR_MSG_1DCOLUMN: str = ...


class ColumnTransformer(TransformerMixin, _BaseComposition):
    n_features_in_: int = ...
    output_indices_: dict = ...
    sparse_output_: bool = ...
    transformers_: list = ...

    _required_parameters: ClassVar[list] = ...

    def __init__(
        self,
        transformers: Sequence[tuple],
        *,
        remainder: Literal["drop", "passthrough", "drop"] | BaseEstimator = "drop",
        sparse_threshold: Float = 0.3,
        n_jobs: None | Int = None,
        transformer_weights: None | dict = None,
        verbose: bool = False,
        verbose_feature_names_out: bool = True,
    ) -> None:
        ...

    def set_output(
        self, *, transform: None | Literal["default", "pandas"] = None
    ) -> BaseEstimator:
        ...

    def get_params(self, deep: bool = True) -> dict:
        ...

    def set_params(self: ColumnTransformer_Self, **kwargs) -> ColumnTransformer_Self:
        ...

    @property
    def named_transformers_(self) -> Bunch:
        ...

    def get_feature_names_out(self, input_features: None | ArrayLike = None) -> ndarray:
        ...

    def fit(
        self: ColumnTransformer_Self, X: MatrixLike, y: None | ArrayLike = None
    ) -> ColumnTransformer_Self:
        ...

    def fit_transform(
        self, X: MatrixLike, y: None | ArrayLike = None
    ) -> ndarray | spmatrix:
        ...

    def transform(self, X: MatrixLike) -> ndarray | spmatrix:
        ...


def make_column_transformer(
    *transformers,
    remainder: Literal["drop", "passthrough", "drop"] | BaseEstimator = "drop",
    sparse_threshold: Float = 0.3,
    n_jobs: None | Int = None,
    verbose: bool = False,
    verbose_feature_names_out: bool = True,
) -> ColumnTransformer:
    ...


class make_column_selector:
    def __init__(
        self,
        pattern: None | str = None,
        *,
        dtype_include: Sequence[dtype] | None | dtype = None,
        dtype_exclude: Sequence[dtype] | None | dtype = None,
    ) -> None:
        ...

    def __call__(self, df: MatrixLike) -> list[str]:
        ...



from numpy.typing import ArrayLike
import itertools
import math

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = [
    "caveman_graph",
    "connected_caveman_graph",
    "relaxed_caveman_graph",
    "random_partition_graph",
    "planted_partition_graph",
    "gaussian_random_partition_graph",
    "ring_of_cliques",
    "windmill_graph",
    "stochastic_block_model",
    "LFR_benchmark_graph",
]

def caveman_graph(l: int, k: int): ...
def connected_caveman_graph(l: int, k: int): ...
@py_random_state(3)
def relaxed_caveman_graph(l: int, k: int, p: float, seed=None): ...
@py_random_state(3)
def random_partition_graph(
    sizes: ArrayLike, p_in: float, p_out: float, seed=None, directed=False
): ...
@py_random_state(4)
def planted_partition_graph(
    l: int, k: int, p_in: float, p_out: float, seed=None, directed=False
): ...
@py_random_state(6)
def gaussian_random_partition_graph(
    n: int, s: float, v: float, p_in: float, p_out: float, directed=False, seed=None
): ...
def ring_of_cliques(num_cliques: int, clique_size: int): ...
def windmill_graph(n: int, k: int): ...
@py_random_state(3)
def stochastic_block_model(
    sizes: ArrayLike,
    p,
    nodelist: ArrayLike | None = None,
    seed=None,
    directed=False,
    selfloops=False,
    sparse=True,
): ...
@py_random_state(11)
def LFR_benchmark_graph(
    n: int,
    tau1: float,
    tau2: float,
    mu: float,
    average_degree: float | None = None,
    min_degree: int | None = None,
    max_degree: int | None = None,
    min_community: int | None = None,
    max_community: int | None = None,
    tol: float = 1.0e-7,
    max_iters: int = 500,
    seed=None,
): ...



def scatter_matrix(
    frame,
    alpha: float = ...,
    figsize=...,
    ax=...,
    grid: bool = ...,
    diagonal: str = ...,
    marker: str = ...,
    density_kwds=...,
    hist_kwds=...,
    range_padding: float = ...,
    **kwds,
): ...
def radviz(frame, class_column, ax=..., color=..., colormap=..., **kwds): ...
def andrews_curves(
    frame, class_column, ax=..., samples: int = ..., color=..., colormap=..., **kwds
): ...
def bootstrap_plot(series, fig=..., size: int = ..., samples: int = ..., **kwds): ...
def parallel_coordinates(
    frame,
    class_column,
    cols=...,
    ax=...,
    color=...,
    use_columns: bool = ...,
    xticks=...,
    colormap=...,
    axvlines: bool = ...,
    axvlines_kwds=...,
    sort_labels: bool = ...,
    **kwds,
): ...
def lag_plot(series, lag: int = ..., ax=..., **kwds): ...
def autocorrelation_plot(series, ax=..., **kwds): ...



from typing import Any, Callable, ClassVar, Literal, TypeVar
from ._base import NeighborsBase, KNeighborsMixin
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Real as Real
from ..utils.metaestimators import available_if as available_if
from ..base import OutlierMixin
from .._typing import Int, MatrixLike, ArrayLike
from ..utils import check_array as check_array
from ..utils.validation import check_is_fitted as check_is_fitted

LocalOutlierFactor_Self = TypeVar("LocalOutlierFactor_Self", bound="LocalOutlierFactor")

# Authors: Nicolas Goix <nicolas.goix@telecom-paristech.fr>
#          Alexandre Gramfort <alexandre.gramfort@telecom-paristech.fr>
# License: BSD 3 clause

import numpy as np
import warnings

__all__ = ["LocalOutlierFactor"]


class LocalOutlierFactor(KNeighborsMixin, OutlierMixin, NeighborsBase):
    n_samples_fit_: int = ...
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    effective_metric_params_: dict = ...
    effective_metric_: str = ...
    offset_: float = ...
    n_neighbors_: int = ...
    negative_outlier_factor_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        n_neighbors: Int = 20,
        *,
        algorithm: Literal["auto", "ball_tree", "kd_tree", "brute", "auto"] = "auto",
        leaf_size: Int = 30,
        metric: str | Callable = "minkowski",
        p: Int = 2,
        metric_params: None | dict = None,
        contamination: float | str = "auto",
        novelty: bool = False,
        n_jobs: None | Int = None,
    ) -> None:
        ...

    def fit_predict(self, X: MatrixLike | ArrayLike, y: Any = None) -> ndarray:
        ...

    def fit(
        self: LocalOutlierFactor_Self, X: MatrixLike, y: Any = None
    ) -> LocalOutlierFactor_Self:
        ...

    def predict(self, X: None | MatrixLike | ArrayLike = None) -> ndarray:
        ...

    def decision_function(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...

    def score_samples(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from ..classes.graph import Graph
from ..classes.graph import Graph

__all__ = [
    "laplacian_spectrum",
    "adjacency_spectrum",
    "modularity_spectrum",
    "normalized_laplacian_spectrum",
    "bethe_hessian_spectrum",
]

def laplacian_spectrum(G: Graph, weight="weight"): ...
def normalized_laplacian_spectrum(G: Graph, weight="weight"): ...
def adjacency_spectrum(G: Graph, weight="weight"): ...
def modularity_spectrum(G: Graph): ...
def bethe_hessian_spectrum(G: Graph, r: float | None = None): ...



from ..classes.graph import Graph

from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = [
    "is_valid_joint_degree",
    "is_valid_directed_joint_degree",
    "joint_degree_graph",
    "directed_joint_degree_graph",
]

def is_valid_joint_degree(joint_degrees) -> bool: ...
@py_random_state(1)
def joint_degree_graph(joint_degrees, seed=None) -> Graph: ...
def is_valid_directed_joint_degree(in_degrees, out_degrees, nkk) -> bool: ...
@py_random_state(3)
def directed_joint_degree_graph(in_degrees, out_degrees, nkk, seed=None) -> Graph: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
from .base_camera import BaseCamera
from .perspective import PerspectiveCamera
from .panzoom import PanZoomCamera
from .arcball import ArcballCamera
from .turntable import TurntableCamera
from .fly import FlyCamera

def make_camera(cam_type: str, *args, **kwargs): ...



from typing import Any

from django.db.migrations.state import ProjectState
from django.db.models.fields import Field

def is_referenced_by_foreign_key(
    state: ProjectState, model_name_lower: str, field: Field[Any, Any], field_name: str
) -> bool: ...



from django.contrib.gis.utils.layermapping import LayerMapError as LayerMapError
from django.contrib.gis.utils.layermapping import LayerMapping as LayerMapping
from django.contrib.gis.utils.ogrinspect import mapping as mapping
from django.contrib.gis.utils.ogrinspect import ogrinspect as ogrinspect



import numpy as np
from typing import Literal, Sequence
from ._typing import *
from .cm import ScalarMappable
from .backend_bases import MouseEvent, RendererBase
from .figure import Figure
from .colors import Colormap, Normalize
from .transforms import Bbox
from .axes import Axes
from .artist import Artist, allow_rasterization
from .transforms import Affine2D
from PIL.Image import Image

interpolations_names = ...

def composite_images(
    images: list[Image], renderer: RendererBase, magnification: float = 1
) -> tuple[np.ndarray, tuple[float, float]]: ...

class _ImageBase(Artist, ScalarMappable):

    zorder = ...
    def __init__(
        self,
        ax: Axes,
        cmap: Colormap = ...,
        norm: Normalize = ...,
        interpolation=...,
        origin=...,
        filternorm=...,
        filterrad=...,
        resample=...,
        *,
        interpolation_stage=...,
        **kwargs
    ) -> None: ...
    def __str__(self) -> str: ...
    def __getstate__(self): ...
    def get_size(self): ...
    def set_alpha(self, alpha: float | ArrayLike | None)-> None: ...
    def changed(self)-> bool: ...
    def make_image(
        self, renderer, magnification=..., unsampled=...
    ) -> tuple[np.ndarray, tuple[float, float], Affine2D]: ...
    @allow_rasterization
    def draw(self, renderer: RendererBase, *args, **kwargs)-> None: ...
    def contains(self, mouseevent: MouseEvent)-> bool: ...
    def write_png(self, fname: str)-> None: ...
    def set_data(self, A: ArrayLike | Image)-> None: ...
    def set_array(self, A: ArrayLike)-> None: ...
    def get_interpolation(self) -> str: ...
    def set_interpolation(
        self,
        s: Literal[
            "antialiased",
            "nearest",
            "bilinear",
            "bicubic",
            "spline16",
            "spline36",
            "hanning",
            "hamming",
            "hermite",
            "kaiser",
            "quadric",
            "catrom",
            "gaussian",
            "bessel",
            "mitchell",
            "sinc",
            "lanczos",
            "none",
        ]
        | None,
    )-> None: ...
    def set_interpolation_stage(self, s: Literal["data", "rgba"] | None)-> None: ...
    def can_composite(self) -> bool: ...
    def set_resample(self, v: bool | None)-> None: ...
    def get_resample(self) -> bool: ...
    def set_filternorm(self, filternorm: bool)-> None: ...
    def get_filternorm(self) -> bool: ...
    def set_filterrad(self, filterrad: float)-> None: ...
    def get_filterrad(self) -> float: ...

class AxesImage(_ImageBase):
    def __init__(
        self,
        ax: Axes,
        cmap: str | Colormap = ...,
        norm: Normalize = ...,
        interpolation: str = ...,
        origin: Literal["upper", "lower"] = ...,
        extent: tuple = ...,
        filternorm: bool = True,
        filterrad: float = 4,
        resample: bool = False,
        *,
        interpolation_stage: Literal["data", "rgba"] = ...,
        **kwargs
    ) -> None: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def make_image(
        self, renderer: RendererBase, magnification: float = ..., unsampled=...
    ): ...
    def set_extent(self, extent: Sequence[float])-> None: ...
    def get_extent(self) -> tuple[float, float, float, float]: ...
    def get_cursor_data(self, event: MouseEvent): ...

class NonUniformImage(AxesImage):
    mouseover = ...
    def __init__(
        self,
        ax: Axes,
        *,
        interpolation: Literal["nearest", "bilinear"] = "nearest",
        **kwargs
    ) -> None: ...
    def make_image(
        self, renderer: RendererBase, magnification: float = ..., unsampled=...
    ): ...
    def set_data(self, x: ArrayLike, y: ArrayLike, A: ArrayLike)-> None: ...
    def set_array(self, *args)-> None: ...
    def set_interpolation(self, s: Literal["nearest", "bilinear"] | None)-> None: ...
    def get_extent(self): ...
    def set_filternorm(self, s)-> None: ...
    def set_filterrad(self, s)-> None: ...
    def set_norm(self, norm: Normalize | None)-> None: ...
    def set_cmap(self, cmap: Colormap | str | None)-> None: ...

class PcolorImage(AxesImage):
    def __init__(
        self,
        ax: Axes,
        x: ArrayLike = ...,
        y: ArrayLike = ...,
        A: ArrayLike = ...,
        cmap: str | Colormap = ...,
        norm: Normalize = ...,
        **kwargs
    ) -> None: ...
    def make_image(
        self, renderer: RendererBase, magnification: float = ..., unsampled=...
    ): ...
    def set_data(self, x: ArrayLike, y: ArrayLike, A: ArrayLike)-> None: ...
    def set_array(self, *args)-> None: ...
    def get_cursor_data(self, event: MouseEvent): ...

class FigureImage(_ImageBase):

    zorder = ...

    def __init__(
        self,
        fig: Figure,
        cmap: Colormap = ...,
        norm: Normalize = ...,
        offsetx=...,
        offsety=...,
        origin=...,
        **kwargs
    ) -> None: ...
    def get_extent(self) -> tuple[float, float, float, float]: ...
    def make_image(
        self, renderer: RendererBase, magnification: float = ..., unsampled=...
    ): ...
    def set_data(self, A)-> None: ...

class BboxImage(_ImageBase):
    def __init__(
        self,
        bbox: Bbox,
        cmap: Colormap = ...,
        norm: Normalize = ...,
        interpolation=...,
        origin=...,
        filternorm=...,
        filterrad=...,
        resample=...,
        **kwargs
    ) -> None: ...
    def get_window_extent(
        self, renderer: RendererBase = ...
    ) -> tuple[float, float, float, float]: ...
    def contains(self, mouseevent: MouseEvent) -> bool: ...
    def make_image(
        self, renderer: RendererBase, magnification: float = ..., unsampled=...
    ): ...

def imread(fname: str | FileLike, format: str = ...) -> np.ndarray: ...
def imsave(
    fname: str | PathLike | FileLike,
    arr: ArrayLike,
    vmin: float = ...,
    vmax: float = ...,
    cmap: str | Colormap = ...,
    format: str = ...,
    origin: Literal["upper", "lower"] = ...,
    dpi: float = ...,
    *,
    metadata: dict = ...,
    pil_kwargs: dict = ...
): ...
def pil_to_array(pilImage) -> np.ndarray: ...
def thumbnail(
    infile: str | FileLike,
    thumbfile: str | FileLike,
    scale: float = ...,
    interpolation: str = ...,
    preview: bool = ...,
) -> Figure: ...



from typing import Callable
from types import MethodType as MethodType
from functools import wraps as wraps, update_wrapper as update_wrapper


class _AvailableIfDescriptor:
    def __init__(self, fn: Callable, check: Callable, attribute_name: str) -> None:
        ...

    def __get__(self, obj, owner=None) -> Callable:
        ...


def available_if(check: Callable) -> Callable:
    ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.signal.sigtools, version: unspecified
import typing
import builtins as _mod_builtins

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
def _convolve2d(in1, in2, flip, mode, boundary, fillvalue) -> typing.Any:
    'out = _convolve2d(in1, in2, flip, mode, boundary, fillvalue)'
    ...

def _correlateND(a, kernel, mode) -> typing.Any:
    "out = _correlateND(a,kernel,mode) \n\n   mode = 0 - 'valid', 1 - 'same', \n  2 - 'full' (default)"
    ...

def _linear_filter() -> typing.Any:
    '(y,Vf) = _linear_filter(b,a,X,Dim=-1,Vi=None)  implemented using Direct Form II transposed flow diagram. If Vi is not given, Vf is not returned.'
    ...

def _medfilt2d() -> typing.Any:
    'filt = _median2d(data, size)'
    ...

def _order_filterND(a, domain, order) -> typing.Any:
    'out = _order_filterND(a,domain,order)'
    ...

def _remez(numtaps, bands, des, weight, type, fs, maxiter, grid_density) -> typing.Any:
    'h = _remez(numtaps, bands, des, weight, type, fs, maxiter, grid_density)\n  returns the optimal (in the Chebyshev/minimax sense) FIR filter impulse\n  response given a set of band edges, the desired response on those bands,\n  and the weight given to the error in those bands.  Bands is a monotonic\n  vector with band edges given in frequency domain where fs is the sampling\n  frequency.'
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import ClassVar, Literal, Optional
from typing_extensions import TypedDict as TypedDict

# =============================================================================
# Federal University of Rio Grande do Sul (UFRGS)
# Connectionist Artificial Intelligence Laboratory (LIAC)
# Renato de Pontes Pereira - rppereira@inf.ufrgs.br
# =============================================================================
# Copyright (c) 2011 Renato de Pontes Pereira, renato.ppontes at gmail dot com
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
# =============================================================================

__author__: Literal["Renato de Pontes Pereira, Matthias Feurer, Joel Nothman"] = ...
__author_email__: str = ...
__version__: str = ...

import re
import csv

# CONSTANTS ===================================================================
_SIMPLE_TYPES: list = ...

_TK_DESCRIPTION: str = ...
_TK_COMMENT: str = ...
_TK_RELATION: str = ...
_TK_ATTRIBUTE: str = ...
_TK_DATA: str = ...

_RE_RELATION = ...
_RE_ATTRIBUTE = ...
_RE_QUOTE_CHARS = ...
_RE_ESCAPE_CHARS = ...
_RE_SPARSE_LINE = ...
_RE_NONTRIVIAL_DATA = ...

ArffDenseDataType = ...
ArffSparseDataType = ...


_ESCAPE_SUB_MAP: dict = ...
_UNESCAPE_SUB_MAP: dict = ...


DENSE: int = ...  # Constant value representing a dense matrix
COO: int = ...  # Constant value representing a sparse matrix in coordinate format
LOD: int = ...  # Constant value representing a sparse matrix in list of
# dictionaries format
DENSE_GEN: int = ...  # Generator of dictionaries
LOD_GEN: int = ...  # Generator of dictionaries
_SUPPORTED_DATA_STRUCTURES: list = ...


# EXCEPTIONS ==================================================================
class ArffException(Exception):
    message: ClassVar[Optional[str]] = ...

    def __init__(self) -> None:
        ...

    def __str__(self) -> str:
        ...


class BadRelationFormat(ArffException):
    message: ClassVar[Literal["Bad @RELATION format, at line %d."]] = ...


class BadAttributeFormat(ArffException):
    message: ClassVar[Literal["Bad @ATTRIBUTE format, at line %d."]] = ...


class BadDataFormat(ArffException):
    def __init__(self, value) -> None:
        ...


class BadAttributeType(ArffException):
    message: ClassVar[Literal["Bad @ATTRIBUTE type, at line %d."]] = ...


class BadAttributeName(ArffException):
    def __init__(self, value, value2) -> None:
        ...


class BadNominalValue(ArffException):
    def __init__(self, value) -> None:
        ...


class BadNominalFormatting(ArffException):
    def __init__(self, value) -> None:
        ...


class BadNumericalValue(ArffException):
    message: ClassVar[Literal["Invalid numerical value, at line %d."]] = ...


class BadStringValue(ArffException):
    message: ClassVar[str] = ...


class BadLayout(ArffException):
    message: ClassVar[Literal["Invalid layout of the ARFF file, at line %d."]] = ...

    def __init__(self, msg: str = "") -> None:
        ...


class BadObject(ArffException):
    def __init__(self, msg: str = "Invalid object.") -> None:
        ...

    def __str__(self) -> str:
        ...


def encode_string(s):
    ...


class EncodedNominalConversor:
    def __init__(self, values) -> None:
        ...

    def __call__(self, value):
        ...


class NominalConversor:
    def __init__(self, values) -> None:
        ...

    def __call__(self, value):
        ...


class DenseGeneratorData:
    def decode_rows(self, stream, conversors):
        ...

    def encode_data(self, data, attributes):
        ...


class _DataListMixin:
    def decode_rows(self, stream, conversors):
        ...


class Data(_DataListMixin, DenseGeneratorData):
    ...


class COOData:
    def decode_rows(self, stream, conversors):
        ...

    def encode_data(self, data, attributes):
        ...


class LODGeneratorData:
    def decode_rows(self, stream, conversors):
        ...

    def encode_data(self, data, attributes):
        ...


class LODData(_DataListMixin, LODGeneratorData):
    ...


# =============================================================================

# ADVANCED INTERFACE ==========================================================
class ArffDecoder:
    def __init__(self) -> None:
        ...

    def decode(self, s, encode_nominal: bool = False, return_type=...):
        ...


class ArffEncoder:
    def encode(self, obj):
        ...

    def iter_encode(self, obj):
        ...


# =============================================================================

# BASIC INTERFACE =============================================================
def load(fp, encode_nominal: bool = False, return_type=...):
    ...


def loads(s, encode_nominal: bool = False, return_type=...):
    ...


def dump(obj, fp):
    ...


def dumps(obj):
    ...


# =============================================================================



"""
This type stub file was generated by pyright.
"""

"""Miscellaneous context managers.
"""
class preserve_keys:
    """Preserve a set of keys in a dictionary.

    Upon entering the context manager the current values of the keys
    will be saved. Upon exiting, the dictionary will be updated to
    restore the original value of the preserved keys. Preserved keys
    which did not exist when entering the context manager will be
    deleted.

    Examples
    --------

    >>> d = {'a': 1, 'b': 2, 'c': 3}
    >>> with preserve_keys(d, 'b', 'c', 'd'):
    ...     del d['a']
    ...     del d['b']      # will be reset to 2
    ...     d['c'] = None   # will be reset to 3
    ...     d['d'] = 4      # will be deleted
    ...     d['e'] = 5
    ...     print(sorted(d.items()))
    ...
    [('c', None), ('d', 4), ('e', 5)]
    >>> print(sorted(d.items()))
    [('b', 2), ('c', 3), ('e', 5)]
    """
    def __init__(self, dictionary, *keys) -> None:
        ...
    
    def __enter__(self): # -> None:
        ...
    
    def __exit__(self, *exc_info): # -> None:
        ...
    





from .asyn_fluid import asyn_fluidc as asyn_fluidc
from .centrality import girvan_newman as girvan_newman
from .kclique import k_clique_communities as k_clique_communities
from .kernighan_lin import kernighan_lin_bisection as kernighan_lin_bisection
from .label_propagation import (
    label_propagation_communities as label_propagation_communities,
    asyn_lpa_communities as label_propagation_communities,
    asyn_lpa_communities as asyn_lpa_communities,
)
from .lukes import lukes_partitioning as lukes_partitioning
from .modularity_max import (
    greedy_modularity_communities as greedy_modularity_communities,
    naive_greedy_modularity_communities as naive_greedy_modularity_communities,
)
from .quality import (
    coverage as coverage,
    modularity as modularity,
    performance as performance,
    partition_quality as partition_quality,
)
from .community_utils import is_partition as is_partition
from .louvain import (
    louvain_communities as louvain_communities,
    louvain_partitions as louvain_partitions,
)



from typing import Any, Callable, Literal, Sequence
from ._typing import *
from .transforms import Transform
from .axis import Axis

class ScaleBase:
    def __init__(self, axis: Axis) -> None: ...
    def get_transform(self) -> Transform: ...
    def set_default_locators_and_formatters(self, axis: Axis): ...
    def limit_range_for_scale(self, vmin: float, vmax: float, minpos: float): ...

class LinearScale(ScaleBase):

    name = ...
    def __init__(self, axis: Axis) -> None: ...
    def set_default_locators_and_formatters(self, axis: Axis): ...
    def get_transform(self) -> Transform: ...

class FuncTransform(Transform):

    input_dims = ...
    def __init__(self, forward: Callable, inverse: Callable) -> None: ...
    def transform_non_affine(self, values: ArrayLike) -> list: ...
    def inverted(self): ...

class FuncScale(ScaleBase):

    name = ...
    def __init__(self, axis: Axis, functions: Sequence[Callable]) -> None: ...
    def get_transform(self) -> FuncTransform: ...
    def set_default_locators_and_formatters(self, axis): ...

class LogTransform(Transform):
    input_dims = ...
    def __init__(self, base, nonpositive: Literal["clip", "mask"] = "clip") -> None: ...
    def __str__(self) -> str: ...
    def transform_non_affine(self, a) -> list: ...
    def inverted(self): ...

class InvertedLogTransform(Transform):
    input_dims = ...
    def __init__(self, base) -> None: ...
    def __str__(self) -> str: ...
    def transform_non_affine(self, a) -> list: ...
    def inverted(self): ...

class LogScale(ScaleBase):

    name = ...
    def __init__(
        self,
        axis: Axis,
        *,
        base: float = 10,
        subs=Sequence[int],
        nonpositive: Literal["clip", "mask"] = "clip"
    ) -> None: ...
    base = ...
    def set_default_locators_and_formatters(self, axis: Axis): ...
    def get_transform(self) -> LogTransform: ...
    def limit_range_for_scale(self, vmin: float, vmax: float, minpos: float): ...

class FuncScaleLog(LogScale):

    name = ...
    def __init__(
        self, axis: Axis, functions: Sequence[Callable], base: float = 10
    ) -> None: ...
    @property
    def base(self): ...
    def get_transform(self) -> Transform: ...

class SymmetricalLogTransform(Transform):
    input_dims = ...
    def __init__(self, base, linthresh, linscale) -> None: ...
    def transform_non_affine(self, a) -> list: ...
    def inverted(self): ...

class InvertedSymmetricalLogTransform(Transform):
    input_dims = ...
    def __init__(self, base, linthresh, linscale) -> None: ...
    def transform_non_affine(self, a) -> list: ...
    def inverted(self): ...

class SymmetricalLogScale(ScaleBase):

    name = ...
    def __init__(
        self,
        axis: Axis,
        *,
        base: float = 10,
        linthresh: float = 2,
        subs: Sequence[int] = ...,
        linscale: float = ...
    ) -> None: ...

    base = ...
    linthresh = ...
    linscale = ...
    def set_default_locators_and_formatters(self, axis: Axis): ...
    def get_transform(self) -> SymmetricalLogTransform: ...

class AsinhTransform(Transform):

    input_dims = ...
    def __init__(self, linear_width) -> None: ...
    def transform_non_affine(self, a) -> list: ...
    def inverted(self): ...

class InvertedAsinhTransform(Transform):

    input_dims = ...
    def __init__(self, linear_width) -> None: ...
    def transform_non_affine(self, a) -> list: ...
    def inverted(self): ...

class AsinhScale(ScaleBase):

    name = ...
    auto_tick_multipliers = ...
    def __init__(
        self,
        axis: Axis,
        *,
        linear_width: float = 1,
        base: float = 10,
        subs: Sequence[int] = ...,
        **kwargs
    ) -> None: ...
    linear_width = ...
    def get_transform(self): ...
    def set_default_locators_and_formatters(self, axis): ...

class LogitTransform(Transform):
    input_dims = ...
    def __init__(self, nonpositive: Literal["mask", "clip"] = ...) -> None: ...
    def transform_non_affine(self, a): ...
    def inverted(self): ...
    def __str__(self) -> str: ...

class LogisticTransform(Transform):
    input_dims = ...
    def __init__(self, nonpositive: Literal["mask", "clip"] = ...) -> None: ...
    def transform_non_affine(self, a): ...
    def inverted(self): ...
    def __str__(self) -> str: ...

class LogitScale(ScaleBase):

    name = ...
    def __init__(
        self,
        axis: Axis,
        nonpositive: Literal["mask", "clip"] = ...,
        *,
        one_half: str = ...,
        use_overline=...
    ) -> None: ...
    def get_transform(self) -> LogitTransform: ...
    def set_default_locators_and_formatters(self, axis: Axis): ...
    def limit_range_for_scale(self, vmin: float, vmax: float, minpos: float): ...

def get_scale_names(): ...
def scale_factory(
    scale: Literal[
        "asinh", "function", "functionlog", "linear", "log", "logit", "symlog"
    ],
    axis: Axis,
    **kwargs
): ...
def register_scale(scale_class): ...



from ...classes.digraph import DiGraph

from ...classes.graph import Graph
from ...utils import pairwise

__all__ = ["global_reaching_centrality", "local_reaching_centrality"]

def global_reaching_centrality(G: DiGraph, weight=None, normalized=True) -> float: ...
def local_reaching_centrality(
    G: DiGraph, v, paths=None, weight=None, normalized=True
) -> float: ...



from matplotlib.gridspec import GridSpec, SubplotSpec
from matplotlib.figure import Figure

class SubplotBase:
    def __init__(self, fig: Figure, *args, **kwargs) -> None: ...
    def get_subplotspec(self) -> SubplotSpec: ...
    def set_subplotspec(self, subplotspec: SubplotSpec)-> None: ...
    def get_gridspec(self) -> GridSpec: ...
    def label_outer(self) -> None: ...

subplot_class_factory = ...
Subplot = ...



from ..classes.multidigraph import MultiDiGraph
import networkx
from ..utils import open_file
from ..classes.graph import Graph

def write_p2g(G: Graph, path, encoding="utf-8"): ...
def read_p2g(path, encoding="utf-8") -> MultiDiGraph: ...
def parse_p2g(lines) -> MultiDiGraph: ...
